from drsai.modules.baseagent.drsaiagent import DrSaiAgent
from drsai.modules.baseagent.user_proxy import DrSaiUserProxyAgent

from autogen_agentchat.base import (
    ChatAgent,
    Response,
    Team,
    TerminatedException,
    TerminationCondition,
    AndTerminationCondition,
    TerminationCondition,
    TaskResult,
    TaskRunner,
    Handoff,
    ) 

from autogen_agentchat.agents import (
    BaseChatAgent,
    AssistantAgent,
    CodeExecutorAgent,
    SocietyOfMindAgent,
    UserProxyAgent,
    MessageFilterAgent,
    MessageFilterConfig,
    PerSourceFilter,
)

from autogen_core.code_executor import CodeBlock, CodeExecutor, CodeResult
from autogen_ext.tools.code_execution import PythonCodeExecutionTool
from autogen_ext.code_executors._common import(
    CommandLineCodeResult,
    _to_code,
    _import_to_str,
    build_python_functions_file,
    to_stub,
    get_file_name_from_content,
    silence_pip,
    get_required_packages,
    lang_to_cmd,
    infer_lang,
)
from autogen_ext.code_executors.docker._docker_code_executor import (
    DockerCommandLineCodeExecutor,
    DockerCommandLineCodeExecutorConfig
)
# from autogen_ext.code_executors.jupyter import (
#     JupyterCodeExecutor, 
#     JupyterCodeResult
# )
# from autogen_ext.code_executors.docker_jupyter import (
#     DockerJupyterCodeExecutor,
#     DockerJupyterServer,
#     JupyterClient,
#     JupyterKernelClient,
#     DockerJupyterCodeResult,
# )
from autogen_ext.code_executors.local import (
    LocalCommandLineCodeExecutor,
    LocalCommandLineCodeExecutorConfig
    )

# autogen_agentchat Messages
from autogen_core.models import (
    AssistantMessage,
    ChatCompletionTokenLogprob,
    CreateResult,
    FinishReasons,
    FunctionExecutionResult,
    FunctionExecutionResultMessage,
    LLMMessage,
    RequestUsage,
    SystemMessage,
    TopLogprob,
    UserMessage,
)
from autogen_agentchat.base import Handoff as HandoffBase


# __all__ = [
#     "DrSaiAgent",
#     "DrSaiUserProxyAgent",

#     "ChatAgent",
#     "Response",
#     "Team",
#     "TerminatedException",
#     "TerminationCondition",
#     "AndTerminationCondition",
#     "OrTerminationCondition",
#     "TaskResult",
#     "TaskRunner",
#     "Handoff",

#     "BaseChatAgent",
#     "AssistantAgent",
#     "CodeExecutorAgent",
#     "SocietyOfMindAgent",
#     "UserProxyAgent",
#     "MessageFilterAgent",
#     "MessageFilterConfig",
#     "PerSourceFilter",

#     "AssistantMessage",
#     "ChatCompletionTokenLogprob",
#     "CreateResult",
#     "FinishReasons",
#     "FunctionExecutionResult",
#     "FunctionExecutionResultMessage",
#     "LLMMessage",
#     "SystemMessage",
#     "RequestUsage",
#     "TopLogprob",
#     "UserMessage",
#     "HandoffBase",
#     ]


"""
session_manager.py — 微信会话状态管理
======================================
管理微信用户 <-> chat_id (agent 实例) 的映射关系，
并持久化到 WECHAT_DIR/sessions.json。

数据结构（sessions.json）：
{
    "counter": 3,
    "sessions": {
        "session_1": {
            "owner": "ilink_user_id_xxx",
            "created_at": 1700000000.0,
            "last_active": 1700000060.0
        }
    },
    "current": {
        "ilink_user_id_xxx": "session_1"
    }
}
"""

import json
import os
import time


class SessionManager:
    """
    管理微信用户与 chat_id 的会话映射。

    - 每个微信用户（by ilink_user_id）在同一时刻有一个"当前 session"
    - 用户可以通过命令新建或切换 session
    - 所有状态持久化到 sessions.json，重启后可恢复
    """

    def __init__(self, sessions_file: str):
        self._file = sessions_file
        self._counter: int = 0
        self._sessions: dict[str, dict] = {}   # chat_id -> {owner, created_at, last_active}
        self._current: dict[str, str] = {}     # user_id -> chat_id
        self.load()

    # ── 持久化 ────────────────────────────────────────────────────────────────

    def load(self) -> None:
        """从文件加载状态，文件不存在则初始化空状态。"""
        if not os.path.exists(self._file):
            return
        try:
            with open(self._file, encoding="utf-8") as f:
                data = json.load(f)
            self._counter = data.get("counter", 0)
            self._sessions = data.get("sessions", {})
            self._current = data.get("current", {})
        except (json.JSONDecodeError, OSError):
            pass  # 文件损坏时从空状态开始

    def save(self) -> None:
        """将当前状态写入文件。"""
        os.makedirs(os.path.dirname(self._file), exist_ok=True)
        data = {
            "counter": self._counter,
            "sessions": self._sessions,
            "current": self._current,
        }
        with open(self._file, "w", encoding="utf-8") as f:
            json.dump(data, f, indent=2, ensure_ascii=False)

    # ── 查询 ──────────────────────────────────────────────────────────────────

    def get_current(self, user_id: str) -> str | None:
        """返回用户当前绑定的 chat_id，未绑定返回 None。"""
        return self._current.get(user_id)

    def list_sessions(self, user_id: str) -> list[str]:
        """返回该用户拥有的所有 chat_id 列表（按创建时间排序）。"""
        owned = [
            (cid, info)
            for cid, info in self._sessions.items()
            if info.get("owner") == user_id
        ]
        owned.sort(key=lambda x: x[1].get("created_at", 0))
        return [cid for cid, _ in owned]

    def get_idle(self, idle_seconds: int) -> list[str]:
        """返回超过 idle_seconds 未活跃的 chat_id 列表。"""
        now = time.time()
        return [
            cid
            for cid, info in self._sessions.items()
            if (now - info.get("last_active", 0)) > idle_seconds
        ]

    # ── 变更 ──────────────────────────────────────────────────────────────────

    def new_session(self, user_id: str) -> str:
        """
        为用户创建新 session，返回新 chat_id（格式：session_{n}）。
        同时将该用户的当前 session 切换到新建的。
        """
        self._counter += 1
        chat_id = f"session_{self._counter}"
        now = time.time()
        self._sessions[chat_id] = {
            "owner": user_id,
            "created_at": now,
            "last_active": now,
        }
        self._current[user_id] = chat_id
        self.save()
        return chat_id

    def get_or_create_session(self, user_id: str) -> str:
        """
        返回用户当前 session；若无则自动新建。
        首次与 Bot 对话时调用。
        """
        chat_id = self.get_current(user_id)
        if chat_id is None or chat_id not in self._sessions:
            chat_id = self.new_session(user_id)
        return chat_id

    def switch_session(self, user_id: str, chat_id: str) -> bool:
        """
        切换到指定 chat_id。
        返回 True 表示成功，False 表示 chat_id 不存在或不属于该用户。
        """
        info = self._sessions.get(chat_id)
        if info is None or info.get("owner") != user_id:
            return False
        self._current[user_id] = chat_id
        self.save()
        return True

    def touch(self, chat_id: str) -> None:
        """更新 chat_id 的最后活跃时间（每次收发消息后调用）。"""
        if chat_id in self._sessions:
            self._sessions[chat_id]["last_active"] = time.time()
            self.save()



from .ag_base_group_chat import AGBaseGroupChatManager, AGGroupChat
from .ag_round_robin_group_chat import AGRoundRobinGroupChatManager, AGRoundRobinGroupChat
from .ag_roundrobin_orchestrator import RoundRobinGroupChat, RoundRobinGroupChatManager
from .ag_swarm_group_chat import AGSwarm, AGSwarmGroupChatManager
from .ag_selector_group_chat import AGSelectorGroupChat, AGSelectorGroupChatManager
from .base_group_chat_runner import BaseGroupChatRunner
from .drsai_base_group_chat_runner import DrSaiBaseGroupChatRunner, DrSaiBaseGroupChatRunnerConfig
from .drsai_base_agent_container import DrSaiChatAgentContainer

from autogen_agentchat.conditions import (
    ExternalTermination,
    HandoffTermination,
    MaxMessageTermination,
    SourceMatchTermination,
    StopMessageTermination,
    TextMentionTermination,
    TimeoutTermination,
    TokenUsageTermination,)

from autogen_agentchat.teams._group_chat._sequential_routed_agent import SequentialRoutedAgent
from autogen_core import (
    AgentId,
    AgentRuntime,
    AgentType,
    SingleThreadedAgentRuntime,
    TypeSubscription,
    AgentInstantiationContext,
    DefaultTopicId,
    MessageContext,
    event,
    rpc,
)

__all__ = [
    "DrSaiChatAgentContainer",
    "AGBaseGroupChatManager",
    "AGGroupChat",
    "RoundRobinGroupChat",
    "RoundRobinGroupChatManager",
    "AGSwarm",
    "AGSwarmGroupChatManager",
    "AGSelectorGroupChat",
    "AGSelectorGroupChatManager",
    "AGRoundRobinGroupChatManager",
    "AGRoundRobinGroupChat",
    "BaseGroupChatRunner",
    "DrSaiBaseGroupChatRunner",
    "DrSaiBaseGroupChatRunnerConfig",
    "ExternalTermination",
    "HandoffTermination",
    "MaxMessageTermination",
    "SourceMatchTermination",
    "StopMessageTermination",
    "TextMentionTermination",
    "TimeoutTermination",
    "TokenUsageTermination",
]


from typing import Dict
from ._anthropic_client import HepAIAnthropicChatCompletionClient
from autogen_core.models import ModelInfo

from .._model_client import ModelFamily

# Mapping of model names to their capabilities
# For Anthropic's Claude models based on:
# https://platform.claude.com/docs/en/about-claude/models/overview
_MODEL_INFO: Dict[str, ModelInfo] = {
    # Claude 4.1 Opus
    "claude-sonnet-4-1": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_4_1_OPUS,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 4.5 Sonnet
    "claude-sonnet-4-5": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_4_5_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 4.5 Haiku
    "claude-haiku-4-5": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_4_5_HAIKU,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 4.5 Opus
    "claude-opus-4-5": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_4_5_OPUS,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 4 Opus
    "claude-opus-4-20250514": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_4_OPUS,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 4 Opus latest alias
    "claude-opus-4-0": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_4_OPUS,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 4 Sonnet
    "claude-sonnet-4-20250514": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_4_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 4 Sonnet latest alias
    "claude-sonnet-4-0": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_4_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 3.7 Sonnet
    "claude-3-7-sonnet-20250219": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_7_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 3.7 Sonnet latest alias
    "claude-3-7-sonnet-latest": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_7_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 3 Opus (most powerful)
    "claude-3-opus-20240229": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_5_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 3 Sonnet (balanced)
    "claude-3-sonnet-20240229": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_5_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 3 Haiku (fastest)
    "claude-3-haiku-20240307": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_5_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 3.5 Sonnet
    "claude-3-5-sonnet-20240620": {
        "vision": True,
        "function_calling": True,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_5_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude Instant v1 (legacy)
    "claude-instant-1.2": {
        "vision": False,
        "function_calling": False,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_5_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 2 (legacy)
    "claude-2.0": {
        "vision": False,
        "function_calling": False,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_5_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
    # Claude 2.1 (legacy)
    "claude-2.1": {
        "vision": False,
        "function_calling": False,
        "json_output": True,
        "family": ModelFamily.CLAUDE_3_5_SONNET,
        "structured_output": False,
        "multiple_system_messages": False,
    },
}

# Model token limits (context window size)
_MODEL_TOKEN_LIMITS: Dict[str, int] = {
    "claude-opus-4-5": 200000,
    "claude-haiku-4-5": 200000,
    "claude-sonnet-4-5": 200000,
    "claude-sonnet-4-1": 200000,
    "claude-3-opus-20240229": 200000,
    "claude-3-sonnet-20240229": 200000,
    "claude-3-haiku-20240307": 200000,
    "claude-3-5-sonnet-20240620": 200000,
    "claude-3-7-sonnet-20250219": 200000,
    "claude-instant-1.2": 100000,
    "claude-2.0": 100000,
    "claude-2.1": 200000,
}


def get_info(model: str) -> ModelInfo:
    """Get the model information for a specific model."""
    # Check for exact match first
    if model in _MODEL_INFO:
        return _MODEL_INFO[model]

    # Check for partial match (for handling model variants)
    for model_id in _MODEL_INFO:
        if model.startswith(model_id.split("-2")[0]):  # Match base name
            return _MODEL_INFO[model_id]

    raise KeyError(f"Model '{model}' not found in model info")


def get_token_limit(model: str) -> int:
    """Get the token limit for a specific model."""
    # Check for exact match first
    if model in _MODEL_TOKEN_LIMITS:
        return _MODEL_TOKEN_LIMITS[model]

    # Check for partial match (for handling model variants)
    for model_id in _MODEL_TOKEN_LIMITS:
        if model.startswith(model_id.split("-2")[0]):  # Match base name
            return _MODEL_TOKEN_LIMITS[model_id]

    # Default to a reasonable limit if model not found
    return 100000


from typing import List, Union, Generator, Iterator, Callable, Awaitable, Optional
from pydantic import BaseModel, Field
import os
import requests
from openai import OpenAI

backend_port = os.environ.get("BACKEND_PORT", "42801")
agnet_name = os.environ.get("AGNET_NAME", "DrSai")

class Pipeline:
    class Valves(BaseModel):

        HEPAI_API_KEY: str = Field(
            default=os.getenv("HEPAI_API_KEY", "your-hepai-api-key-here"),
            description="大模型的服务商的apikey, 默认HepAI平台的api_key",
        )
        HEPAI_BASE_URL: str = Field(
            default="https://aiapi.ihep.ac.cn/apiv2",
            description="大模型的服务商的base_url, 默认HepAI平台的base_url",
        )
        DRSAI_NAME: str = Field(
            default=agnet_name,
            description="你的Dr.Sai智能体后端的名称, 默认为DrSai",
        )
        DRSAI_URL: str = Field(
            default=f"http://localhost:{backend_port}/apiv2",
            description="你的Dr.Sai智能体后端的地址, 默认为http://localhost:42801/apiv2",
        )
        BASE_MODELS: str = Field(
            default="deepseek-ai/deepseek-v3:671b",
            description="drsai后端使用的基座模型名称，默认deepseek-ai/deepseek-v3:671b",
        )

    def __init__(self):
        # Optionally, you can set the id and name of the pipeline.
        # Best practice is to not specify the id so that it can be automatically inferred from the filename, so that users can install multiple versions of the same pipeline.
        # The identifier must be unique across all pipelines.
        # The identifier must be an alphanumeric string that can include underscores or hyphens. It cannot contain spaces, special characters, slashes, or backslashes.
        # self.id = "openai_pipeline"

        self.valves = self.Valves()
        self.user_id = None
        self.user_name = None
        self.user_email = None
        self.message_id = None
        self.chat_id = None
        self.name = self.valves.DRSAI_NAME 
        pass

    async def on_startup(self):
        # This function is called when the server is started.
        print(f"on_startup:{__name__}")
        pass

    async def on_shutdown(self):
        # This function is called when the server is stopped.
        print(f"on_shutdown:{__name__}")
        pass
    
    async def inlet(self, body: dict, user: Optional[dict] = None) -> dict:
        print(f"inlet:{__name__}")
        # print(f"body: {body}")
        # print(f"user: {user}")
        self.user_id = user.get("id")
        self.user_name = user.get("name")
        self.user_email = user.get("email")
        self.chat_id = body.get("metadata").get("chat_id")
        self.message_id = body.get("metadata").get("message_id")
        body["user"] = user
        return body

    def pipe(
        self, user_message: str, 
        model_id: str, messages: List[dict], 
        body: dict, 
        headers: dict = None,
        **kwargs
    ) -> Union[str, Generator, Iterator]:
        
        self.backend_base_url = self.valves.DRSAI_URL # 连接drsai后端的地址
        # 配置hepai平台的api_key和base_url
        self.base_models = self.valves.BASE_MODELS # drsai后端使用的基座模型

        # This is where you can add your custom pipelines like RAG.
        print(f"pipe:{__name__}") 

        # 构建访问openai的请求头和参数
        new_hearers = {}
        if headers:
            new_hearers["Authorization"] = headers["authorization"]
            if new_hearers["Authorization"].split(" ")[-1] == "0p3n-w3bu!":
                new_hearers["Authorization"] = f"Bearer {self.valves.HEPAI_API_KEY}"
        else: 
            new_hearers["Authorization"] = f"Bearer {self.valves.HEPAI_API_KEY}"
        new_hearers["Content-Type"] = "application/json"

        # title_generation任务
        if body["stream"] is False:
            HEPAI_API_KEY = new_hearers["Authorization"].split(" ")[-1]
            self.hepai_client = OpenAI(
                api_key=HEPAI_API_KEY, base_url=self.valves.HEPAI_BASE_URL
            )
            payload = {**body, "model": self.base_models}
            if "user" in payload:
                del payload["user"]
            if "chat_id" in payload:
                del payload["chat_id"]
            if "title" in payload:
                del payload["title"]
            # print(f"payload: {payload}")
        
            try:
                r = requests.post(
                    url=f"{self.valves.HEPAI_BASE_URL}/chat/completions",
                    json=payload,
                    headers=new_hearers,
                    # stream=True,
                )
                r.raise_for_status()
                response = r.iter_lines()
                return response
            except Exception as e:
                return f"Error: {e}"


        # 访问drsai多智能体框架后端接口
        # if user_message == "自动继续": # 适配特殊前端
        #     body["messages"][-1]["content"] = "exit"
        try:
            body["base_models"] = self.base_models
            body["chat_id"] = self.chat_id
            body["message_id"] = self.message_id
            r = requests.post(
                f"{self.backend_base_url}/chat/completions", 
                headers=new_hearers,
                json=body,
                stream=True,
                )
            r.raise_for_status()
            return r.iter_lines()
        except Exception as e:
            return f"Error: {e}"


"""
EdgeAgent Task Planning Module
任务规划与用户交互模块

负责:
1. 分析用户请求是否需要任务规划
2. 生成任务规划(可选调用Agent Skills或知识库)
3. 与前端用户交互获取计划反馈
4. 支持重新规划
"""

from typing import Dict, List, Optional, Any, AsyncGenerator
from pydantic import BaseModel
from loguru import logger
import json

from drsai.modules.managers.messages import (
    TextMessage,
    ModelClientStreamingChunkEvent,
)
from drsai.modules.components.model_client import ChatCompletionClient
from drsai.modules.components.model_context import ChatCompletionContext
from drsai.modules.baseagent import (
    UserMessage,
    AssistantMessage,
    CreateResult,
)
from drsai import CancellationToken


class TaskPlanStep(BaseModel):
    """单个任务步骤"""
    title: str  # 步骤标题
    details: str  # 详细描述
    agent_name: Optional[str] = None  # 执行此步骤的agent名称(如果需要)
    tool_or_skill: Optional[str] = None  # 需要使用的工具或技能


class TaskPlan(BaseModel):
    """任务规划"""
    needs_plan: bool  # 是否需要规划
    response: str  # 对用户的响应
    task: str  # 任务摘要
    plan_summary: str  # 规划摘要
    steps: List[Dict[str, Any]]  # 任务步骤列表


class TaskPlanner:
    """
    任务规划器
    负责分析用户请求、生成任务计划、与用户交互
    """

    def __init__(
        self,
        agent_name: str,
        model_client: ChatCompletionClient,
        skills_description: Optional[str] = None,
        enable_skills_for_planning: bool = True,
    ):
        """
        Args:
            agent_name: Agent名称
            model_client: 模型客户端
            skills_description: 可用的skills描述(用于规划时参考)
            enable_skills_for_planning: 是否在规划时可以调用skills
        """
        self.agent_name = agent_name
        self.model_client = model_client
        self.skills_description = skills_description or ""
        self.enable_skills_for_planning = enable_skills_for_planning

    def get_planning_system_prompt(self, with_skills: bool = False) -> str:
        """
        获取任务规划的系统提示词
        Args:
            with_skills: 是否包含skills信息
        Returns:
            系统提示词
        """
        base_prompt = """You are a task planning expert for scientific data analysis.

Your job is to:
1. Analyze the user's request
2. Determine if task planning is needed (complex multi-step tasks need planning)
3. If planning is needed, decompose the task into clear, executable steps

**When to plan:**
- Multi-step workflows (data processing + analysis + visualization)
- Complex analysis requiring multiple tools/skills
- Tasks needing coordination between different agents

**When NOT to plan:**
- Simple single-step requests (read a file, run a command)
- Direct questions with straightforward answers
- Tasks that are already clear and specific

**Output Format (JSON):**
```json
{
    "needs_plan": true/false,
    "response": "Your analysis and response to user",
    "task": "Brief task summary",
    "plan_summary": "Overview of the approach",
    "steps": [
        {
            "title": "Step 1 title",
            "details": "Detailed description of what to do",
            "agent_name": "AgentName (optional)",
            "tool_or_skill": "tool/skill name (optional)"
        }
    ]
}
```

**Important:**
- Keep steps actionable and specific
- Each step should have a clear outcome
- Consider dependencies between steps
- Mention which tools/skills/agents are needed
"""

        if with_skills and self.skills_description:
            base_prompt += f"""

**Available Agent Skills for Reference:**
{self.skills_description}

You can suggest loading specific skills in the plan if they're relevant.
"""

        return base_prompt

    async def analyze_and_plan(
        self,
        user_request: str,
        model_context: ChatCompletionContext,
        cancellation_token: CancellationToken,
        use_skills_context: bool = False,
    ) -> TaskPlan:
        """
        分析用户请求并生成任务计划

        Args:
            user_request: 用户请求
            model_context: 模型上下文
            cancellation_token: 取消令牌
            use_skills_context: 是否使用skills上下文辅助规划

        Returns:
            TaskPlan对象
        """
        try:
            # 构建规划请求消息
            planning_prompt = f"""User Request: {user_request}

Please analyze this request and provide a task plan in JSON format.
Consider:
1. Is this a complex task that needs planning?
2. What are the main steps required?
3. What tools, skills, or agents are needed?
4. Are there dependencies between steps?

Respond ONLY with valid JSON following the specified format."""

            # 准备消息
            planning_messages = [
                UserMessage(
                    content=self.get_planning_system_prompt(with_skills=use_skills_context),
                    source="system"
                ),
                UserMessage(content=planning_prompt, source="user")
            ]

            # 调用LLM生成计划
            result: CreateResult = await self.model_client.create(
                messages=planning_messages,
                cancellation_token=cancellation_token,
            )

            # 解析结果
            plan_json = self._extract_json_from_response(result.content)
            task_plan = TaskPlan(**plan_json)

            logger.info(f"Generated task plan: needs_plan={task_plan.needs_plan}, "
                       f"steps={len(task_plan.steps)}")

            return task_plan

        except Exception as e:
            logger.error(f"Failed to generate task plan: {e}")
            # 返回一个不需要规划的默认plan
            return TaskPlan(
                needs_plan=False,
                response=f"I'll help you with: {user_request}",
                task=user_request,
                plan_summary="Direct execution without detailed planning",
                steps=[]
            )

    def _extract_json_from_response(self, content: str) -> Dict[str, Any]:
        """
        从LLM响应中提取JSON
        Args:
            content: LLM响应内容
        Returns:
            解析后的JSON字典
        """
        # 尝试直接解析
        try:
            return json.loads(content)
        except json.JSONDecodeError:
            pass

        # 尝试提取代码块中的JSON
        if "```json" in content:
            start = content.find("```json") + 7
            end = content.find("```", start)
            json_str = content[start:end].strip()
            return json.loads(json_str)
        elif "```" in content:
            start = content.find("```") + 3
            end = content.find("```", start)
            json_str = content[start:end].strip()
            return json.loads(json_str)

        # 如果都失败,抛出异常
        raise ValueError(f"Cannot extract JSON from response: {content}")

    async def send_plan_to_user(
        self,
        task_plan: TaskPlan,
        agent_name: str,
    ) -> TextMessage:
        """
        将任务计划发送给用户(前端)

        Args:
            task_plan: TaskPlan对象
            agent_name: Agent名称

        Returns:
            包含计划的TextMessage
        """
        plan_dict = task_plan.model_dump()

        plan_message = TextMessage(
            content=json.dumps(plan_dict, ensure_ascii=False),
            source=agent_name,
            metadata={"internal": "no", "type": "plan_message"},
        )

        logger.info(f"Sent plan to user with {len(task_plan.steps)} steps")
        return plan_message

    def parse_user_feedback(
        self,
        user_message: TextMessage
    ) -> Dict[str, Any]:
        """
        解析用户对计划的反馈

        Args:
            user_message: 用户消息

        Returns:
            包含用户反馈的字典 {accepted: bool, plan: Optional[Dict], feedback: str}
        """
        try:
            # 从metadata中获取用户请求
            user_request = user_message.metadata.get("user_request")
            if not user_request:
                return {"accepted": False, "feedback": "No user feedback found"}

            # 解析用户反馈
            feedback_data = json.loads(user_request)

            result = {
                "accepted": feedback_data.get("accepted", False),
                "plan": feedback_data.get("plan"),  # 修改后的计划
                "feedback": feedback_data.get("feedback", ""),  # 用户的文字反馈
            }

            logger.info(f"Parsed user feedback: accepted={result['accepted']}")
            return result

        except Exception as e:
            logger.error(f"Failed to parse user feedback: {e}")
            return {"accepted": False, "feedback": str(e)}

    async def replan_with_feedback(
        self,
        original_plan: TaskPlan,
        user_feedback: str,
        model_context: ChatCompletionContext,
        cancellation_token: CancellationToken,
    ) -> TaskPlan:
        """
        根据用户反馈重新规划

        Args:
            original_plan: 原始计划
            user_feedback: 用户反馈
            model_context: 模型上下文
            cancellation_token: 取消令牌

        Returns:
            更新后的TaskPlan对象
        """
        try:
            replan_prompt = f"""Original Task Plan:
{json.dumps(original_plan.model_dump(), indent=2, ensure_ascii=False)}

User Feedback:
{user_feedback}

Please revise the task plan based on the user's feedback.
Keep the same JSON format and make appropriate adjustments.

Respond ONLY with valid JSON."""

            # 准备消息
            planning_messages = [
                UserMessage(
                    content=self.get_planning_system_prompt(with_skills=True),
                    source="system"
                ),
                UserMessage(content=replan_prompt, source="user")
            ]

            # 调用LLM重新规划
            result: CreateResult = await self.model_client.create(
                messages=planning_messages,
                cancellation_token=cancellation_token,
            )

            # 解析结果
            plan_json = self._extract_json_from_response(result.content)
            revised_plan = TaskPlan(**plan_json)

            logger.info(f"Revised task plan based on feedback")
            return revised_plan

        except Exception as e:
            logger.error(f"Failed to replan with feedback: {e}")
            # 如果失败,返回原计划
            return original_plan

    async def send_step_execution_status(
        self,
        step_index: int,
        step: Dict[str, Any],
        plan_length: int,
        agent_name: str,
        progress_summary: Optional[str] = None,
    ) -> TextMessage:
        """
        发送当前步骤执行状态给前端

        Args:
            step_index: 步骤索引
            step: 步骤信息
            plan_length: 总步骤数
            agent_name: Agent名称
            progress_summary: 进度摘要

        Returns:
            状态消息
        """
        status_data = {
            "title": step.get("title", f"Step {step_index + 1}"),
            "index": step_index,
            "details": step.get("details", ""),
            "agent_name": step.get("agent_name", agent_name),
            "instruction": step.get("details", ""),
            "progress_summary": progress_summary or f"Executing step {step_index + 1} of {plan_length}",
            "plan_length": plan_length,
        }

        status_message = TextMessage(
            content=json.dumps(status_data, ensure_ascii=False),
            source=agent_name,
            metadata={"internal": "no", "type": "step_execution"},
        )

        logger.info(f"Sent step execution status: step {step_index + 1}/{plan_length}")
        return status_message


if __name__ == "__main__":
    # 简单测试
    import asyncio
    from drsai.modules.components.model_client import HepAIChatCompletionClient, ModelFamily

    async def test_planner():
        # 创建模型客户端
        model_client = HepAIChatCompletionClient(
            model="glm-4-flash",
            model_family=ModelFamily.GLM4,
        )

        # 创建规划器
        planner = TaskPlanner(
            agent_name="TestAgent",
            model_client=model_client,
        )

        # 测试规划
        user_request = "请帮我分析data.csv文件,提取关键统计信息,并生成一个可视化报告"

        from drsai.modules.components.model_context import BufferedChatCompletionContext
        context = BufferedChatCompletionContext(buffer_size=10)

        plan = await planner.analyze_and_plan(
            user_request=user_request,
            model_context=context,
            cancellation_token=CancellationToken(),
        )

        print("Generated Plan:")
        print(f"Needs Plan: {plan.needs_plan}")
        print(f"Task: {plan.task}")
        print(f"Steps: {len(plan.steps)}")
        for i, step in enumerate(plan.steps):
            print(f"  Step {i+1}: {step.get('title')}")

    # asyncio.run(test_planner())
    print("Task planner module created successfully")



from autogen_ext.tools.mcp import StdioServerParams, mcp_server_tools, StdioMcpToolAdapter

async def web_fetch() -> StdioMcpToolAdapter:
    try:
        web_fetch_tools = await mcp_server_tools(
            StdioServerParams(
                command="uv",
                args=["tool", "run", "mcp-server-fetch"],
                env=None) )
    except Exception as e:
        error =  "\nPlease make sure you have installed the uv tool and 'pip install mcp-server-fetch'."
        raise str(e) + error
    return web_fetch_tools[0]



"""
EdgeAgent User Profile Manager Module
用户画像与文件管理模块

管理EdgeAgent的用户特定文件结构:
work_dir/{user_id}/
├── AGENTS.md        # 用户画像和智能体配置
├── Memories/        # 按时间列表储存的记忆(按session组织)
├── Skills.md        # 任务执行历史总结
├── skills/          # 学习到的skills
├── TOOLS.md         # 工具使用偏好
└── USER.md          # 用户画像描述
"""

from pathlib import Path
from typing import Dict, List, Optional, Any
import json
from datetime import datetime
from loguru import logger
from pydantic import BaseModel
from drsai.modules.components.tool import (
    ToolSchema,
    ParametersSchema,
    )

class UserProfile(BaseModel):
    """用户画像数据模型"""
    user_id: str
    user_name: str
    agent_name: Optional[str] = None
    ask_before_plan: bool = False
    created_at: str = ""
    updated_at: str = ""

class TaskStep(BaseModel):
    """单个任务步骤的记录"""
    step_index: int
    step_title: str
    tool_or_skill_used: Optional[str] = None
    action_details: str
    result: str
    error: Optional[str] = None


class SessionMemory(BaseModel):
    """
    会话级别的记忆 - 对应一次完整的用户任务会话
    包含任务规划、执行过程、工具调用、错误修正等完整信息
    """
    session_id: str  # 对应 thread_id
    user_id: str
    start_time: str
    end_time: Optional[str] = None

    # 任务信息
    original_user_request: str  # 用户原始请求
    task_plan: Optional[Dict[str, Any]] = None  # 任务规划结果
    needs_plan: bool = False  # 是否需要规划

    # 执行过程
    execution_steps: List[TaskStep] = []  # 按顺序记录每个步骤

    # 工具和技能使用记录
    tools_used: List[str] = []
    skills_loaded: List[str] = []
    subagents_spawned: List[str] = []

    # 结果和学习
    final_result: Optional[str] = None
    errors_encountered: List[Dict[str, Any]] = []  # 错误及修正过程
    learned_patterns: List[str] = []  # 从本次任务学到的模式

    # 用户反馈
    user_feedback: Optional[Dict[str, Any]] = None

class UserProfileManager:
    """
    管理EdgeAgent用户画像、记忆、技能和偏好设置的文件管理器
    """

    def __init__(
            self, 
            agent_name: str,
            work_dir: str | Path,
            user_id: str,
            thread_id: str,
            user_config: UserProfile | None = None,
            ):
        """
        Args:
            work_dir: 工作目录根路径
            user_id: 用户唯一标识
        """
        
        self.agent_name = agent_name
        self.user_name = user_id
        self.work_dir = Path(work_dir)
        self.work_dir.mkdir(exist_ok=True)
        self.tmp_dir = self.work_dir / "tmp"
        self.tmp_dir.mkdir(exist_ok=True)
        self.download_dir = self.work_dir / "downloads"
        self.download_dir.mkdir(exist_ok=True)
        self.config_path = self.work_dir / "configs"
        self.config_path.mkdir(exist_ok=True)
        self.user_id = user_id
        self.thread_id = thread_id

        # 定义各个文件路径
        self.agents_md = self.config_path / "AGENTS.md" # 整体的系统提示词：用户画像和智能体配置
        self.subagent_config_path = self.config_path / "SUBAGENT_CONFIG.json" # 子智能体配置
        self.memorie_path = self.config_path / "MEMORY.md" # 用户近期的记忆
        self.memories_dir = self.config_path / "memories" # 用户所有的记忆文件
        self.memories_document_ids = self.memories_dir / "document_ids.json" # 记录每个记忆文件的RAGFlow文档ID
        self.skills_md = self.config_path / "SKILLS.md" # 调用技能描述，目前未用
        self.skills_dir = self.config_path / "skills" # 用户的所有
        self.tools_md = self.config_path / "TOOLS.md" # 用户的工作环境配置
        self.tools_config_path = self.config_path / "TOOLS_CONFIG.json" # 工具配置
        self.user_md = self.config_path / "USER.md" # 用户的个人描述
        self.user_config_path = self.config_path / "USER_CONFIG.json" # 用户配置
            
        # user's user profile
        self.first_time_setup = True
        if self.agents_md.exists():
            self.first_time_setup = False

        if not self.first_time_setup:
            self.user_config = self.load_user_config()
        elif user_config:
            self.user_config = user_config
        else:
            self.user_config = None

        # 初始化文件
        self._initialize_files()

    def _initialize_files(self):
        """初始化所有必要的文件"""
 
        if not self.user_config_path.exists():
            self._create_user_config()
        self.agent_name = self.user_config.agent_name
        self.user_name = self.user_config.user_name

        if not self.user_md.exists():
            self._create_user_md()

        if not self.tools_md.exists():
            self._create_tools_md()
        
        if not self.tools_config_path.exists():
            with self.tools_config_path.open("w", encoding='utf-8') as f:
                json.dump([], f, indent=4, ensure_ascii=False)

        if not self.skills_md.exists():
            self._create_skills_md()
        
        if not self.agents_md.exists():
            self._create_agents_md()
        
        if not self.subagent_config_path.exists():
            with self.subagent_config_path.open("w", encoding='utf-8') as f:
                json.dump({}, f, indent=4, ensure_ascii=False)
        
        # 创建目录
        if not self.memories_dir.exists():
            self.memories_dir.mkdir(exist_ok=True)
            with self.memories_document_ids.open("w", encoding='utf-8') as f:
                json.dump({}, f, indent=4, ensure_ascii=False)

        if not self.skills_dir.exists():
            self.skills_dir.mkdir(exist_ok=True)

    def _create_user_config(self):
        """创建用户配置文件"""
        
        if not self.user_config:
            self.user_config = UserProfile(
                user_id=self.user_id,
                user_name=self.user_id,
                agent_name=self.agent_name,
                ask_before_plan=False,
                created_at=datetime.now().isoformat(),
                updated_at=datetime.now().isoformat(),
            )
        with self.user_config_path.open("w", encoding='utf-8') as f:
            json.dump(self.user_config.model_dump(), f, indent=4, ensure_ascii=False)

    def _create_user_md(self):
        """创建USER.md文件"""
        content = f"""# User Profile: {self.user_name}

## Basic Information
- **User ID:** {self.user_id}
- **User Name:** {self.user_name}
- **What does the user call you:** {self.agent_name}
- **Pronouns:** *(optional)*
- **Timezone:** 
- **Notes:** 

## Preferences

*(What do they care about? What projects are they working on? What annoys them? What makes them laugh? Build this over time.)*

[User preferences and the agent's response style. To be filled based on user interactions]

---

The more you know, the better you can help. But remember — you're learning about a person, not building a dossier. Respect the difference.

"""
        self.user_md.write_text(content, encoding='utf-8')

    def _create_tools_md(self):
        """创建TOOLS.md文件"""
        content = f"""# Tool Preferences for User: {self.user_id}

## Environment Setup

### Working Directory: 
    - {self.work_dir}

### Temporary Directory
    - {self.tmp_dir}

### Download Directory
    - {self.download_dir}

### Congiguration Files
    - {self.config_path}/AGENTS.md
    - {self.config_path}/SUBAGENT_CONFIG.json
    - {self.config_path}/SKILLS.md
    - {self.config_path}/TOOLS.md
    - {self.config_path}/TOOLS_CONFIG.json
    - {self.config_path}/USER.md
    - {self.config_path}/USER_CONFIG.json

### Personal Skills and Memory dirs: 
    - {self.config_path}/skills
    - {self.config_path}/memories

### SSH setup: 
    - home-server → 192.168.1.100, user: admin

## Usage Preferences
[To be learned from user interactions]

## Frequently Used Tools and Skills
[To be tracked automatically]

**Note:** 

- Remember to perform operations such as downloading files in the Download Directory, and avoid interfering with the contents of other configured files.
"""
        self.tools_md.write_text(content, encoding='utf-8')

    def _create_agents_md(self):
        """创建AGENTS.md文件"""

        user_md = self.get_user_profile()
        tools_md = self.get_tools_preferences()

        content = f"""# {self.agent_name} Configuration for User: {self.user_name}

## Agent Capabilities

Your name is {self.agent_name} with the following capabilities:

1. **Task Planning & Decomposition**: Analyze user requirements and decompose into executable subtasks
2. **Multi-task Progress Management**: Automatically update task status to prevent information loss
3. **Tool & Skills Invocation**: Proactively load tools, agent skills, and spawn subagents
4. **Learning & Adaptation**: Summarize task execution patterns and save as reusable skills

**Note:** - When the user performs a non-greeting task fisrtly, attempt to retrieve relevant memories using the `retreve_from_memory` tool. 
- If the memory content is relevant, prioritize using the memory content for the reply and inform the user of the time when the memory was recorded, among other information. 
- If the memory content is not relevant, proceed with the following rules.

## Rules:

- Use Skill tool IMMEDIATELY when a task matches a skill description
- Use Task tool for subtasks needing focused exploration or implementation
- Use TodoWrite to track multi-step work
- Prefer tools over prose. Act, don't just explain.
- After finishing, summarize what changed.

## Workflow
1. Receive user task → Analyze if planning is needed
2. If planning needed → Generate plan → Get user approval
3. Execute tasks with progress tracking (TodoManager)
4. Record all actions, tool calls, errors in current session memory
5. Learn from execution → Save skills if requested by user
6. Handle errors → Request user help if blocked


{user_md}

{tools_md}
"""
        self.agents_md.write_text(content, encoding='utf-8')
    
    def _create_skills_md(self):
        """创建Skills.md文件"""
        content = f"""# Learned Skills for User: {self.user_id}

## Task Execution History Summary
[This file summarizes successful task execution patterns]

## Available Custom Skills
[Skills are stored in skills/ directory with SKILL.md format]

## Created: {datetime.now().isoformat()}
## Updated: {datetime.now().isoformat()}
"""
        self.skills_md.write_text(content, encoding='utf-8')

    def get_agent_system_prompt(self) -> str:
        """
        获取AGENTS.md作为系统提示词的一部分
        Returns:
            AGENTS.md的内容
        """
        try:
            return self.agents_md.read_text(encoding='utf-8')
        except Exception as e:
            logger.error(f"Failed to read AGENTS.md: {e}")
            return ""
    
    def load_subagents_config(self) -> dict:
        subagent_config_data = json.loads(self.subagent_config_path.read_text(encoding='utf-8'))
        return subagent_config_data
    
    def load_user_config(self) -> UserProfile:
        config_data = json.loads(self.user_config_path.read_text(encoding='utf-8'))
        return UserProfile(**config_data)

    def get_update_user_profile_tool(strict: bool = False) -> ToolSchema:
        parameters: ParametersSchema = {
            "type": "object",
            "properties": {
                "items": {
                    "type": "array",
                    "minItems": 1,
                    "description": "A list of configuration updates to apply in a single request.",
                    "items": {
                        "type": "object",
                        "properties": {
                            "update_type": {
                                "type": "string",
                                "enum": ["agent", "user", "tool"],
                                "description": (
                                    "The target configuration to update. "
                                    "`agent` updates system/agent instructions, "
                                    "`user` updates user profile settings, "
                                    "`tool` updates tool preferences."
                                ),
                            }
                        },
                        "required": ["update_type"],
                        "additionalProperties": False,
                    },
                }
            },
            "required": ["items"],
            "additionalProperties": False,
        }

        tool_schema: ToolSchema = {
            "name": "UpdateUserProfile",
            "description": (
                "Apply one or more configuration updates in a single call. "
                "Each update specifies its target type (agent, user, or tool) "
            ),
            "parameters": parameters,
            "strict": strict,
        }
        return tool_schema
    
    def get_user_profile(self) -> str:
        """
        获取用户画像信息
        Returns:
            USER.md的内容
        """
        try:
            return self.user_md.read_text(encoding='utf-8')
        except Exception as e:
            logger.error(f"Failed to read USER.md: {e}")
            return ""

    def update_user_profile(self, content: str):
        """
        更新用户画像
        Args:
            content: 新的用户画像内容
        """
        try:
            self.user_md.write_text(content, encoding='utf-8')
            logger.info(f"Updated USER.md for {self.user_id}")
        except Exception as e:
            logger.error(f"Failed to update USER.md: {e}")

    def get_user_config_tool(self, strict: bool = False,) -> ToolSchema:
        """
        Update user profile configuration file based on user requirements, including assistant name, user name, interests, and preferences.

        Args:
            user_name: str  User's name
            agent_name: str  Assistant's name
            ask_before_plan: bool  Whether to show plan before execution
        """
        parameters = ParametersSchema(
            type="object",
            properties={
                "user_name": {
                    "type": "string",
                    "description": "User's name for personalized addressing"
                },
                "agent_name": {
                    "type": "string",
                    "description": "Assistant's name as preferred by the user"
                },
                "ask_before_plan": {
                    "type": "boolean",
                    "description": "Whether to show the plan and ask for user approval before executing complex tasks"
                },
            },
            required=[],  # All fields are optional, allowing partial updates
            additionalProperties=False,
        )
        tool_schema = ToolSchema(
            name="UpdateUserConfig",
            description="""Update user profile configuration including name, and assistant settings.

Use this tool when the user wants to:
- Change their name or how they want to be addressed
- Update the assistant's name
- Configure whether to ask before planning tasks

You can update one or multiple fields at once. Only provide the fields that need to be updated.""",
            parameters=parameters,
            strict=strict,
        )
        return tool_schema
    
    def update_user_config(self, **kwargs) -> str:
        """
        安全地更新用户配置并写入文件

        Args:
            **kwargs: 要更新的UserProfile字段，只接受UserProfile模型中定义的字段

        Raises:
            ValueError: 当传入无效字段、值无效或user_config未初始化时
            IOError: 当文件写入失败时
        """
        if self.user_config is None:
            raise ValueError("User config is not initialized")

        try:
            # 使用model_copy并自动添加updated_at
            kwargs['updated_at'] = datetime.now().isoformat()
            updated_config = self.user_config.model_copy(update=kwargs)

            # 写入文件并更新内存中的配置
            self.user_config_path.write_text(
                updated_config.model_dump_json(indent=4, ensure_ascii=False),
                encoding='utf-8'
            )
            self.user_config = updated_config

            self.agent_name = updated_config.agent_name
            self.user_name = updated_config.user_name

            logger.info(f"Successfully updated user config for {self.user_id}")
            return f"Successfully updated user config for {self.user_id}"

        except Exception as e:
            logger.error(f"Failed to update user config: {e}")
            # raise
            return f"Failed to update user config: {e}"

    def load_user_tools_config(self) -> dict:
        tools_config_data = json.loads(self.tools_config_path.read_text(encoding='utf-8'))
        return tools_config_data
    
    def get_tools_preferences(self) -> str:
        """
        获取工具偏好
        Returns:
            TOOLS.md的内容
        """
        try:
            return self.tools_md.read_text(encoding='utf-8')
        except Exception as e:
            logger.error(f"Failed to read TOOLS.md: {e}")
            return ""

    def update_tools_preferences(self, content: str):
        """
        更新工具偏好
        Args:
            content: 新的工具偏好内容
        """
        try:
            self.tools_md.write_text(content, encoding='utf-8')
            logger.info(f"Updated TOOLS.md for {self.user_id}")
        except Exception as e:
            logger.error(f"Failed to update TOOLS.md: {e}")


    def save_learned_skill(self, skill_name: str, skill_content: str) -> str:
        """
        保存学习到的skill
        Args:
            skill_name: skill名称
            skill_content: skill内容(应该是SKILL.md格式)
        Returns:
            保存的文件路径
        """
        try:
            skill_dir = self.skills_dir / skill_name
            skill_dir.mkdir(exist_ok=True)

            skill_file = skill_dir / "SKILL.md"
            if skill_file.exists():
                with skill_file.open("a", encoding='utf-8') as f:
                    f.write("\n" + skill_content)
            else:
                skill_file.write_text(skill_content, encoding='utf-8')

            # 更新Skills.md
            self._update_skills_summary(skill_name)

            logger.info(f"Saved skill '{skill_name}' to {skill_file}")
            return str(skill_file)
        except Exception as e:
            logger.error(f"Failed to save skill: {e}")
            return ""

    def _update_skills_summary(self, skill_name: str):
        """更新Skills.md,添加新的skill记录"""
        try:
            content = self.skills_md.read_text(encoding='utf-8')
            timestamp = datetime.now().isoformat()
            new_entry = f"\n## {skill_name}\n- Added: {timestamp}\n- Location: skills/{skill_name}/SKILL.md\n"

            # 在文件末尾添加新条目
            content += new_entry
            self.skills_md.write_text(content, encoding='utf-8')
        except Exception as e:
            logger.error(f"Failed to update skills summary: {e}")

    def update_agents_config(self, new_content: str):
        """
        更新AGENTS.md配置
        Args:
            new_content: 新的配置内容
        """
        try:
            self.agents_md.write_text(new_content, encoding='utf-8')
            logger.info(f"Updated AGENTS.md for {self.user_id}")
        except Exception as e:
            logger.error(f"Failed to update AGENTS.md: {e}")

    def save_session_memory(self, session_memory: list[dict]):
        """
        保存会话记忆到Memories/目录
        Args:
            session_memory: list[dict]
        Returns:
            保存的文件路径
        """
        
        try:
            filename = f"session_{self.thread_id}.json"
            filepath = self.memories_dir / filename
            if filepath.exists():
                existing = json.loads(filepath.read_text(encoding='utf-8'))
                existing.extend(session_memory)
                session_memory = existing
            filepath.write_text(
                json.dumps(session_memory, indent=4, ensure_ascii=False),
                encoding='utf-8'
            )
            logger.info(f"Saved session memory to {filepath}")
            return str(filepath)
        except Exception as e:
            logger.error(f"Failed to save session memory: {e}")
    
    def update_document_ids(self, thread_id: str, document_id: str):
         
        memories_document_ids = json.loads(self.memories_document_ids.read_text(encoding='utf-8'))
        memories_document_ids[thread_id] = document_id
        self.memories_document_ids.write_text(
            json.dumps(memories_document_ids, indent=4, ensure_ascii=False),
            encoding='utf-8'
        )
    
    def get_document_ids(self, thread_id: str) -> str|None:
        """
        获取document_id
        Args:
            thread_id: 会话ID
        Returns:
            document_id
        """
        memories_document_ids = json.loads(self.memories_document_ids.read_text(encoding='utf-8'))
        return memories_document_ids.get(thread_id)

    # def create_session_memory(
    #     self,
    #     session_id: str,
    #     user_request: str,
    #     needs_plan: bool = False
    # ) -> SessionMemory:
    #     """
    #     创建一个新的会话记忆对象
    #     Args:
    #         session_id: 会话ID (对应thread_id)
    #         user_request: 用户原始请求
    #         needs_plan: 是否需要任务规划
    #     Returns:
    #         SessionMemory对象
    #     """
    #     return SessionMemory(
    #         session_id=session_id,
    #         user_id=self.user_id,
    #         start_time=datetime.now().isoformat(),
    #         original_user_request=user_request,
    #         needs_plan=needs_plan
    #     )

    # def save_session_memory(self, memory: SessionMemory) -> str:
    #     """
    #     保存会话记忆到Memories/目录
    #     Args:
    #         memory: SessionMemory对象
    #     Returns:
    #         保存的文件路径
    #     """
    #     try:
    #         timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
    #         filename = f"session_{timestamp}_{memory.session_id}.json"
    #         filepath = self.memories_dir / filename

    #         filepath.write_text(
    #             memory.model_dump_json(indent=2),
    #             encoding='utf-8'
    #         )
    #         logger.info(f"Saved session memory to {filepath}")
    #         return str(filepath)
    #     except Exception as e:
    #         logger.error(f"Failed to save session memory: {e}")
    #         return ""

    # def load_session_memory(self, session_id: str) -> Optional[SessionMemory]:
    #     """
    #     加载指定session的记忆
    #     Args:
    #         session_id: 会话ID
    #     Returns:
    #         SessionMemory对象,如果不存在返回None
    #     """
    #     try:
    #         # 查找包含session_id的文件
    #         memory_files = list(self.memories_dir.glob(f"session_*_{session_id}.json"))
    #         if not memory_files:
    #             return None

    #         # 取最新的一个
    #         latest_file = sorted(memory_files, reverse=True)[0]
    #         content = json.loads(latest_file.read_text(encoding='utf-8'))
    #         return SessionMemory(**content)
    #     except Exception as e:
    #         logger.error(f"Failed to load session memory: {e}")
    #         return None

    # def search_memories(
    #     self,
    #     keywords: Optional[List[str]] = None,
    #     limit: int = 5
    # ) -> List[SessionMemory]:
    #     """
    #     搜索历史会话记忆
    #     Args:
    #         keywords: 搜索关键词列表
    #         limit: 返回结果数量限制
    #     Returns:
    #         SessionMemory对象列表
    #     """
    #     try:
    #         memories = []
    #         memory_files = sorted(
    #             self.memories_dir.glob("session_*.json"),
    #             reverse=True
    #         )[:limit * 2]  # 获取更多文件以便过滤

    #         for file in memory_files:
    #             try:
    #                 content = json.loads(file.read_text(encoding='utf-8'))
    #                 memory = SessionMemory(**content)

    #                 # 如果没有关键词,直接添加
    #                 if not keywords:
    #                     memories.append(memory)
    #                 else:
    #                     # 检查关键词是否在用户请求、任务步骤或结果中
    #                     searchable_text = f"{memory.original_user_request} "
    #                     searchable_text += " ".join([step.action_details for step in memory.execution_steps])
    #                     if memory.final_result:
    #                         searchable_text += f" {memory.final_result}"

    #                     if any(kw.lower() in searchable_text.lower() for kw in keywords):
    #                         memories.append(memory)

    #                 if len(memories) >= limit:
    #                     break
    #             except Exception as e:
    #                 logger.warning(f"Failed to load memory {file}: {e}")
    #                 continue

    #         return memories
    #     except Exception as e:
    #         logger.error(f"Failed to search memories: {e}")
    #         return []

    # def get_recent_memories(self, count: int = 5) -> List[SessionMemory]:
    #     """
    #     获取最近的会话记忆
    #     Args:
    #         count: 返回数量
    #     Returns:
    #         SessionMemory对象列表
    #     """
    #     return self.search_memories(limit=count)

    # def get_all_user_context(self) -> str:
    #     """
    #     获取所有用户上下文信息的摘要
    #     Returns:
    #         包含所有关键信息的字符串
    #     """
    #     context = []

    #     # AGENTS配置
    #     agents_content = self.get_agent_system_prompt()
    #     if agents_content:
    #         context.append("=== Agent Configuration ===")
    #         context.append(agents_content)

    #     # 用户画像
    #     user_content = self.get_user_profile()
    #     if user_content:
    #         context.append("\n=== User Profile ===")
    #         context.append(user_content)

    #     # 工具偏好
    #     tools_content = self.get_tools_preferences()
    #     if tools_content:
    #         context.append("\n=== Tool Preferences ===")
    #         context.append(tools_content)

    #     # 最近的记忆(简要)
    #     recent_memories = self.get_recent_memories(count=3)
    #     if recent_memories:
    #         context.append("\n=== Recent Session Memories (Last 3) ===")
    #         for mem in recent_memories:
    #             context.append(f"- Session {mem.session_id} [{mem.start_time}]:")
    #             context.append(f"  User Request: {mem.original_user_request[:150]}...")
    #             if mem.final_result:
    #                 context.append(f"  Result: {mem.final_result[:100]}...")
    #             context.append(f"  Tools Used: {', '.join(mem.tools_used)}")
    #             if mem.learned_patterns:
    #                 context.append(f"  Learned: {', '.join(mem.learned_patterns)}")

    #     return "\n".join(context)


# if __name__ == "__main__":
#     # 测试代码
#     manager = UserProfileManager("/tmp/test_edgeagent", "test_user_001")

#     # 测试创建会话记忆
#     session_memory = manager.create_session_memory(
#         session_id="thread_12345",
#         user_request="请帮我分析这个数据文件,提取关键信息并生成报告",
#         needs_plan=True
#     )

#     # 模拟添加任务规划
#     session_memory.task_plan = {
#         "needs_plan": True,
#         "steps": [
#             {"title": "读取数据文件", "details": "使用read工具读取文件内容"},
#             {"title": "数据分析", "details": "调用数据分析子智能体"},
#             {"title": "生成报告", "details": "整合结果并生成文字报告"}
#         ]
#     }

#     # 模拟添加执行步骤
#     session_memory.execution_steps.append(TaskStep(
#         step_index=0,
#         step_title="读取数据文件",
#         tool_or_skill_used="run_read",
#         action_details="读取文件 data.csv, 共1000行",
#         result="成功读取数据"
#     ))

#     session_memory.tools_used = ["run_read", "run_bash"]
#     session_memory.skills_loaded = ["data_analysis_basics"]
#     session_memory.end_time = datetime.now().isoformat()
#     session_memory.final_result = "任务完成,报告已生成"

#     # 保存会话记忆
#     manager.save_session_memory(session_memory)

#     # 测试搜索记忆
#     memories = manager.search_memories(keywords=["数据", "分析"])
#     print(f"Found {len(memories)} session memories")

#     # 测试获取完整上下文
#     context = manager.get_all_user_context()
#     print(context[:500])



from .wechat_login import login_wechat_main, load_credentials, save_credentials
from .wechat_client import AsyncWeChatAPI, load_sync_buf, save_sync_buf, split_text
from .session_manager import SessionManager
from .wechat_bot import WeChatBot
from .idle_monitor import idle_monitor

__all__ = [
    "login_wechat_main",
    "load_credentials",
    "save_credentials",
    "AsyncWeChatAPI",
    "load_sync_buf",
    "save_sync_buf",
    "split_text",
    "SessionManager",
    "WeChatBot",
    "idle_monitor",
]



from abc import ABC, abstractmethod


from pydantic import BaseModel
from typing import List, Dict, Any
from typing_extensions import Self
from autogen_core._component_config import Component, ComponentBase
from autogen_core import (
    CancellationToken, 
    ComponentModel,
    Component
    )
from autogen_core.model_context import ChatCompletionContext

class BaseLearningConfig(BaseModel):
    learning_prompt: str
    model_client: ComponentModel
    db_manager: ComponentModel|None = None


class BaseLearning(ABC, ComponentBase[BaseModel]):

    component_type = "learning"

    @abstractmethod
    async def learn_from_context(
        self,
        model_context: ChatCompletionContext,
        learning_config: BaseLearningConfig,
        cancellation_token: CancellationToken | None = None,
        **kwargs: Any,
    ) -> Any:
        """
        Retrieve data from the sensor.

        Args:
            serson_config: The configuration for the sensor.
            cancellation_token: A cancellation token to cancel the operation.

        Returns:
            Any: The retrieved data.
        """
        ...

    @abstractmethod
    async def update_context(
        self,
        model_context: ChatCompletionContext,
    ) -> Any:
        """
        Update the provided model context using retrieved data content.

        Args:
            model_context: The context to update.

        Returns:
            Any: The updated model context.
        """
        ...
    
    @abstractmethod
    async def clear(self) -> None:
        """Clear all entries from learning"""
        ...

    @abstractmethod
    async def close(self) -> None:
        """Clean up any resources used by the learning implementation."""
        ...



from typing import (
    AsyncGenerator, 
    List, 
    Sequence, 
    Dict, 
    Any, 
    Callable, 
    Awaitable, 
    Union, 
    Optional, 
    Tuple,
    Self,
    Mapping,
    )
import os, json, sys, uuid
import asyncio
from pydantic import BaseModel
from pathlib import Path
from loguru import logger

from drsai import CancellationToken, FunctionCall
from drsai.modules.baseagent import (
    DrSaiAgent, 
    HandoffBase, 
    Response, 
    TaskResult, 
    CreateResult,
    FunctionExecutionResultMessage,
    AssistantMessage,
    UserMessage,
    SystemMessage,
    LLMMessage
    )
from drsai.modules.baseagent.drsaiagent import DrSaiAgentConfig
from drsai.modules.baseagent import CodeExecutorAgent, CodeExecutor
from drsai.modules.components import (
    ComponentModel,
)
from drsai.modules.components.model_client import ChatCompletionClient
from drsai.modules.components.model_context import (
    ChatCompletionContext,
)
from drsai.modules.components.memory import Memory
from drsai.modules.components.tool import (
    BaseTool, 
    FunctionTool, 
    Workbench,
    ToolSchema,
    ParametersSchema,
    )
from drsai.modules.managers.messages import (
    BaseAgentEvent,
    BaseChatMessage,
    AgentEvent,
    ChatMessage,
    HandoffMessage,
    MemoryQueryEvent,
    ModelClientStreamingChunkEvent,
    TextMessage,
    StopMessage,
    ToolCallExecutionEvent,
    ToolCallRequestEvent,
    ToolCallSummaryMessage,
    UserInputRequestedEvent,
    ThoughtEvent,
    AgentLogEvent,
    
    StructuredMessage,
    StructuredMessageFactory,
    # MultiModalMessage,
    Image,
)
from drsai.modules.managers.database import DatabaseManager
from drsai.modules.components.skills import SkillLoader
from .managers.operater_funs import get_operator_funcs
from .managers.todo_manager import TodoManager
from drsai.configs.constant import RUNS_DIR

class SkillAgentConfig(DrSaiAgentConfig):
    skills_dir: str
    executor: ComponentModel
    work_dir: str
    sub_agent_config: Dict | None

class SkillAgent(DrSaiAgent):

    def __init__(
        self,
        name: str,
        *,
        model_client: ChatCompletionClient = None,
        tools: List[BaseTool[Any, Any] | Callable[..., Any] | Callable[..., Awaitable[Any]]] | None = None,
        workbench: Workbench | None = None,
        handoffs: List[HandoffBase | str] | None = None,
        model_context: ChatCompletionContext | None = None,
        description: str = "An agent that provides assistance with ability to use tools.",
        system_message: (
            str | None
        ) = "You are a helpful AI assistant. Solve tasks using your tools. Reply with TERMINATE when the task has been completed.",
        model_client_stream: bool = True,
        reflect_on_tool_use: bool | None = None,
        tool_call_summary_format: str = "{result}",
        tool_call_summary_prompt: str| None = None,
        output_content_type: type[BaseModel] | None = None,
        output_content_type_format: str | None = None,
        memory: Sequence[Memory] | None = None,
        metadata: Dict[str, str] | None = None,

        # drsaiAgent specific
        skills_dir: str | None = None,
        skills_loader: SkillLoader | None = None,
        work_dir: str | None = None,
        executor: CodeExecutor | None = None,
        sub_agent_config: Dict | None = None,
        max_turn_count: int = 20,
        memory_function: Callable | None = None,
        reply_function: Callable | None = None,
        db_manager: DatabaseManager | None = None,
        thread_id: str | None = None,
        user_id: str | None = None,
    ):
        super().__init__(
            name=name,
            model_client=model_client,
            tools=tools,
            workbench=workbench,
            handoffs=handoffs,
            model_context=model_context,
            description=description,
            system_message=system_message,
            model_client_stream=model_client_stream,
            reflect_on_tool_use=reflect_on_tool_use,
            tool_call_summary_format=tool_call_summary_format,
            tool_call_summary_prompt=tool_call_summary_prompt,
            output_content_type=output_content_type,
            output_content_type_format=output_content_type_format,
            memory=memory,
            metadata=metadata,
            memory_function=memory_function,
            reply_function=reply_function,
            db_manager=db_manager,
            thread_id=thread_id,
            user_id=user_id,
        )
        self._work_dir = work_dir
        if not self._work_dir:
            self._work_dir = Path(RUNS_DIR) / self._thread_id / self._user_id
            self._work_dir.mkdir(parents=True, exist_ok=True)

        # basic tools
        self._basic_funcs: List[Callable] = get_operator_funcs(work_dir)
        self._basic_funcs_names = [func.__name__ for func in self._basic_funcs]
        for func in self._basic_funcs:
            self._tools.append(FunctionTool(func, description=func.__doc__))
        self._workbench._tools = self._tools

        # skills
        if skills_loader:
            self._skills_loader = skills_loader
        else:
            self._skills_loader = SkillLoader(skills_dir=skills_dir)
        if self._skills_loader.skills:
            self._agent_skills_tools = [self.get_agent_skills_tools()]
        else:
            self._agent_skills_tools = []

        # executor
        self._executor = executor
        
        # sub_agent_config
        self._sub_agent_config = sub_agent_config
        self._subagent_descriptions  = self.get_subagent_descriptions()
        self._subagent_tools = [self.get_subagent_tools()]

        # max_turn_count    
        self._max_turn_count = max_turn_count

        #  todo manager
        self._todo_manager = TodoManager()
        self._todo_tools = [self.get_todo_manager_tools()]

    def get_agent_skills_tools(self, strict: bool = False,) -> ToolSchema:
        """Get the skills' tools available to this agent."""
        
        parameters = ParametersSchema(
            type="object",
            properties={
                "skill": {
                        "type": "string",
                        "description": "Name of the skill to load"
                    }
            },
            required=["skill"],
            additionalProperties=False,
        )
        tool_schema = ToolSchema(
            name="Skill",
            description=f"""Load a skill to gain specialized knowledge for a task.

Available skills:
{self._skills_loader.get_descriptions()}

When to use:
- IMMEDIATELY when user task matches a skill description
- Before attempting domain-specific work (PDF, MCP, etc.)

The skill content will be injected into the conversation, giving you
detailed instructions and access to resources.""",
            parameters=parameters,
            strict=strict,
        )
        
        return tool_schema
    
    def get_todo_manager_tools(self, strict: bool = False,) -> ToolSchema:
        parameters = ParametersSchema(
            type="object",
            properties={
                "items": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "properties": {
                            "content": {"type": "string"},
                            "status": {
                                "type": "string",
                                "enum": ["pending", "in_progress", "completed"]
                            },
                            # "activeForm": {"type": "string"},
                        },
                    },
                },
            },
            required=["content", "status"], # , "activeForm"
            additionalProperties=False,
        )
        tool_schema = ToolSchema(
            name="TodoWrite",
            description="Create/Update task list.",
            parameters=parameters,
            strict=strict,
        )
        return tool_schema
    
    def get_subagent_tools(self, strict: bool = False,) -> ToolSchema:
        parameters = ParametersSchema(
            type="object",
            properties={
                "description": {
                    "type": "string",
                    "description":  "Short task description (3-5 words)"
                },
                "prompt": {
                    "type": "string",
                    "description": "The specific tasks that need to be executed by the sub agent. If the tasks include code blocks, files, etc. that need to be executed, they must be filled in completely."
                },
                "agent_type": {
                    "type": "string",
                    "enum": list(self._sub_agent_config.keys())
                },
            },
            required=["description", "prompt", "agent_type"],
            additionalProperties=False,
        )
        tool_schema = ToolSchema(
            name="Task",
            description=f"Spawn a subagent for a focused subtask.\n\nAgent types:\n{self._subagent_descriptions}",
            parameters=parameters,
            strict=strict,
        )
        return tool_schema
    
    def get_subagent_descriptions(self) -> str:
        """Generate agent type descriptions for system prompt."""
        return "\n".join(
            f"- {name}: {cfg['description']}"
            for name, cfg in self._sub_agent_config.items()
        )
    async def on_messages_stream(
        self, messages: Sequence[BaseChatMessage], cancellation_token: CancellationToken
    ) -> AsyncGenerator[BaseAgentEvent | BaseChatMessage | Response, None]:
        """
        Process the incoming messages with the assistant agent and yield events/responses as they happen.
        """
        # TODO: check attached_files
        attached_files_str = messages[-1].metadata.get("attached_files")
        if attached_files_str:
            attached_files = json.loads(attached_files_str)

        # monitor the pause event
        if self.is_paused:
            yield Response(
                chat_message=TextMessage(
                    content=f"The {self.name} is paused.",
                    source=self.name,
                    metadata={"internal": "yes"},
                )
            )
            return

        # Set up background task to monitor the pause event and cancel the task if paused.
        async def monitor_pause() -> None:
            await self._paused.wait()
            self.is_paused = True

        monitor_pause_task = asyncio.create_task(monitor_pause())
        inner_messages: List[BaseAgentEvent | BaseChatMessage] = []
        try:
            # Gather all relevant state here
            agent_name = self.name
            model_context = self._model_context
            memory = self._memory
            system_messages = self._system_messages
            workbench = self._workbench
            handoff_tools = self._handoff_tools
            handoffs = self._handoffs
            manager_tools = self._agent_skills_tools+self._todo_tools+self._subagent_tools
            model_client = self._model_client
            model_client_stream = self._model_client_stream
            reflect_on_tool_use = self._reflect_on_tool_use
            tool_call_summary_format = self._tool_call_summary_format
            output_content_type = self._output_content_type
            format_string = self._output_content_type_format
            
            # Add new user/handoff messages to the model context
            await self._add_messages_to_context(
                model_context=model_context,
                messages=messages,
            )

            # TODO: Update model context with any relevant memory -> When? How?

            turn_count = 0
            while turn_count < self._max_turn_count:
                
                model_result = None
                async for inference_output in self._call_llm(
                    model_client=model_client,
                    model_client_stream=model_client_stream,
                    system_messages=system_messages,
                    model_context=model_context,
                    workbench=workbench,
                    handoff_tools=handoff_tools,
                    manager_tools=manager_tools,
                    agent_name=agent_name,
                    cancellation_token=cancellation_token,
                    output_content_type=output_content_type,
                ):
                    if self.is_paused:
                        raise asyncio.CancelledError()
                    
                    if isinstance(inference_output, CreateResult):
                        model_result = inference_output
                    else:
                        # Streaming chunk event
                        yield inference_output

                assert model_result is not None, "No model result was produced."

                # --- NEW: If the model produced a hidden "thought," yield it as an event ---
                if model_result.thought:
                    thought_event = ThoughtEvent(content=model_result.thought, source=agent_name)
                    yield thought_event
                    inner_messages.append(thought_event)

                # Add the assistant message to the model context (including thought if present)
                await model_context.add_message(
                    AssistantMessage(
                        content=model_result.content,
                        source=agent_name,
                        thought=getattr(model_result, "thought", None),
                    )
                )
                
                # If direct text response (string)
                if isinstance(model_result.content, str):
                    reponse = await self.handle_str_reponse(
                            model_result = model_result,
                            agent_name = agent_name,
                            format_string = format_string,
                            inner_messages = inner_messages,
                            output_content_type = output_content_type,)
                    
                    yield reponse
                    return

                # Otherwise, we have function calls
                assert isinstance(model_result.content, list) and all(
                    isinstance(item, FunctionCall) for item in model_result.content
                )
                
                # Log ToolCallRequestEvent
                tool_call_msg = ToolCallRequestEvent(
                    content=model_result.content,
                    source=agent_name,
                    models_usage=model_result.usage,
                )
                logger.debug(tool_call_msg)
                tools_name = [tool.name for tool in model_result.content] 
                yield AgentLogEvent(
                    title="I am using tools: " + " ".join(tools_name),
                    source=agent_name, 
                    content=str(tool_call_msg.content), 
                    content_type="tools")
                inner_messages.append(tool_call_msg)
                yield tool_call_msg

                # handle tool call
                for i in range(len(model_result.content)):
                    argument = json.loads(model_result.content[i].arguments)
                    tool_name = model_result.content[i].name
                    if tool_name == "TodoWrite":
                        async for message in self.handle_todo_write(
                            argument = argument,
                            agent_name = agent_name, 
                            model_context = model_context):
                            if isinstance(message, StopMessage):
                                yield message
                                return
                            yield message
                    elif tool_name == "Task":
                        async for message in self.handle_subagent_repsonse(
                            agent_name = agent_name,
                            model_client = model_client,
                            model_client_stream = model_client_stream,
                            model_context = model_context,
                            argument = argument,
                            cancellation_token = cancellation_token,
                            output_content_type = output_content_type,
                        ):
                            if isinstance(message, StopMessage):
                                yield message
                                return
                            yield message
                    elif tool_name == "Skill":
                        skill_content = self._skills_loader.run_skill(argument["skill"])
                        await model_context.add_message(
                            UserMessage(
                                content=f"Skill for {argument["skill"]}: {skill_content}",
                                source="user",
                            )
                        )
                    elif tool_name in self._basic_funcs_names:
                        async for message in self._process_model_result(
                            model_result=model_result,
                            inner_messages=inner_messages,
                            cancellation_token=cancellation_token,
                            agent_name=agent_name,
                            system_messages=system_messages,
                            model_context=model_context,
                            workbench=workbench,
                            handoff_tools=handoff_tools,
                            handoffs=handoffs,
                            model_client=model_client,
                            model_client_stream=model_client_stream,
                            reflect_on_tool_use=reflect_on_tool_use,
                            tool_call_summary_format=tool_call_summary_format,
                            tool_call_summary_prompt=self._tool_call_summary_prompt,
                            output_content_type=output_content_type,
                            format_string=format_string,
                        ):
                            if self.is_paused:
                                raise asyncio.CancelledError()
                            if isinstance(message, Response):
                                yield message.chat_message
                                # repetitive addition
                                # await model_context.add_message(
                                #     UserMessage(
                                #         content=message.chat_message.content,
                                #         source="user",
                                #     )
                                # )
                            else:
                                yield message
                    else:
                        await model_context.add_message(
                            UserMessage(
                                content=f"Unknown tool: {argument["name"]}",
                                source="user",
                            )
                        )

                turn_count += 1

        except asyncio.CancelledError:
            # If the task is cancelled, we respond with a message.
            yield Response(
                chat_message=TextMessage(
                    content="The task was cancelled by the user.",
                    source=self.name,
                    metadata={"internal": "yes"},
                ),
                inner_messages=inner_messages,
            )
        except Exception as e:
            # logger.error(f"Error in {self.name}: {e}")
            logger.exception(f"Error in {self.name}")
            # add to chat history
            await model_context.add_message(
                AssistantMessage(
                    content=f"An error occurred while executing the task: {e}",
                    source=self.name
                )
            )
            yield Response(
                chat_message=TextMessage(
                    content=f"An error occurred while executing the task: {e}",
                    source=self.name,
                    metadata={"internal": "no"},
                ),
                inner_messages=inner_messages,
            )
        finally:
            # Cancel the monitor task.
            try:
                monitor_pause_task.cancel()
                await monitor_pause_task
            except asyncio.CancelledError:
                pass

    async def _call_llm(
        self,
        model_client: ChatCompletionClient,
        model_client_stream: bool,
        system_messages: List[SystemMessage],
        model_context: ChatCompletionContext,
        workbench: Workbench,
        handoff_tools: List[BaseTool[Any, Any]],
        manager_tools: List[ToolSchema],
        agent_name: str,
        cancellation_token: CancellationToken,
        output_content_type: type[BaseModel] | None,
    ) -> AsyncGenerator[Union[CreateResult, ModelClientStreamingChunkEvent], None]:
        """
        Perform a model inference and yield either streaming chunk events or the final CreateResult.
        """
        all_messages = await model_context.get_messages()
        
        llm_messages: List[LLMMessage] = self._get_compatible_context(model_client=model_client, messages=system_messages + all_messages)

        # 自定义的memory_function，用于RAG检索等功能，为大模型回复增加最新的知识
        if self._memory_function is not None:
            llm_messages = await self._call_memory_function(llm_messages, model_client, cancellation_token, agent_name)

        all_tools = (await workbench.list_tools()) + handoff_tools + manager_tools
        # model_result: Optional[CreateResult] = None
        if self._reply_function is not None:
            # 自定义的reply_function，用于自定义对话回复的定制
            async for chunk in self._call_reply_function(
                llm_messages, 
                model_client = model_client, 
                workbench=workbench,
                handoff_tools=handoff_tools,
                tools = all_tools,
                agent_name=agent_name, 
                cancellation_token=cancellation_token,
                db_manager=self._db_manager,
            ):
                # if isinstance(chunk, CreateResult):
                #     model_result = chunk
                yield chunk
        else:
           async for chunk in self.call_llm(
                agent_name = agent_name,
                model_client = model_client,
                llm_messages = llm_messages, 
                tools = all_tools, 
                model_client_stream = model_client_stream,
                cancellation_token = cancellation_token,
                output_content_type = output_content_type,
           ):
               yield chunk
    
    async def handle_str_reponse(
            self,
            model_result: CreateResult,
            agent_name: str,
            format_string: str | None,
            inner_messages: List[BaseAgentEvent | BaseChatMessage],
            output_content_type: type[BaseModel] | None,
    ) -> Response:

        if output_content_type:
            content = output_content_type.model_validate_json(model_result.content)
            return Response(
                chat_message=StructuredMessage[output_content_type](  # type: ignore[valid-type]
                    content=content,
                    source=agent_name,
                    models_usage=model_result.usage,
                    format_string=format_string,
                ),
                inner_messages=inner_messages,
            )
        else:
            return Response(
                chat_message=TextMessage(
                    content=model_result.content,
                    source=agent_name,
                    models_usage=model_result.usage,
                ),
                inner_messages=inner_messages,
            )
    
    def get_tools_for_agent(self, agent_type: str) -> list:
        """Filter tools based on agent type."""
        allowed = self._sub_agent_config.get(agent_type, {}).get("tools", "*")
        if allowed == "*":
            return self._tools
        return [t for t in self._tools if t.name in allowed]

    # TODO: handle Task
    async def handle_subagent_repsonse(
        self,
        agent_name: str,
        model_client: ChatCompletionClient,
        model_client_stream: bool,
        model_context: ChatCompletionContext,
        argument:Dict[str, Any],
        cancellation_token: CancellationToken,
        output_content_type: type[BaseModel] | None,
    ) -> AsyncGenerator[BaseAgentEvent | BaseChatMessage, None]:
        """
        Sub agent can actuate the sub task.

        The types of sub agent:
        1. code_executor
        2. normal drsai agent
        """
        try:
            description, prompt, agent_type = argument["description"], argument["prompt"], argument["agent_type"]

            # get sub agent system prompt
            sub_system = f"""You are a {agent_type} subagent at {self._work_dir}.

    {self._sub_agent_config[agent_type].get("prompt", "")}

    Complete the task and return a clear, concise summary."""
            
            # construct task messages
            task_messages: Sequence[BaseChatMessage] = []
            task_messages.append(TextMessage(content=f"Current task: \n\n{prompt}", source="user"))
            llm_messages = await model_context.get_messages()
            backgroud_message = "Below are the historical chat records between the user and various intelligent assistants, which can be referenced when executing the current task.\n\n"
            for llm_message in llm_messages:
                if isinstance(llm_message, UserMessage) or isinstance(llm_message, AssistantMessage):
                    backgroud_message += f"{llm_message.source}: {llm_message.content}\n\n"
            task_messages.append(TextMessage(content=backgroud_message, source="user"))

            #  TODO: handle turn count fro multi-turn task.
            # turn_count = 0
            # while turn_count < self._max_turn_count:
            #     turn_count += 1
        
            if agent_type == "coder_executor":
                subagent = CodeExecutorAgent(
                    name=agent_type,
                    code_executor=self._executor,
                    model_client_stream=model_client_stream,
                )
            else:
                tools = self.get_tools_for_agent(agent_type)
                description=self._sub_agent_config[agent_type].get("description", "")
                subagent = DrSaiAgent(
                    name=agent_type,
                    system_message=sub_system,
                    description=description,
                    tools=tools,
                    model_client=model_client,
                    model_client_stream=model_client_stream,
                    output_content_type=output_content_type,)
            
            async for message in subagent.on_messages_stream(messages=task_messages, cancellation_token=cancellation_token):
                if isinstance(message, Response):
                    yield message.chat_message
                    await model_context.add_message(
                        UserMessage(
                            content=message.chat_message.content,
                            source="user",
                        )
                    )
                    return
                yield message

        except Exception as e:
            logger.exception(f"Error in {self.name}")
            yield ModelClientStreamingChunkEvent(
                content=str(e)+"\n\n",
                source=self.name,
            )
            await model_context.add_message(
                UserMessage(
                    content=str(e),
                    source="user",
                )
            )
            yield StopMessage(
                content=str(e),
                source=agent_name,
            )

    # TODO: handle TodoWrite
    async def handle_todo_write(
        self,
        argument: Dict[str, Any],
        agent_name: str,
        model_context: ChatCompletionContext,
    ) -> AsyncGenerator[BaseAgentEvent | BaseChatMessage, None]:
        try:
            todo_list = self._todo_manager.update(argument["items"])
            # send stream message
            yield ModelClientStreamingChunkEvent(
                content=todo_list+"\n\n",
                source=self.name,
            )
            # add message to model_context with user source
            await model_context.add_message(
                UserMessage(
                    content=self._todo_manager.get_task_prompt(),
                    source="user",
                )
            )
            # send text message to save to db in drsai ui
            yield TextMessage(
                content=todo_list,
                source=agent_name,
            )
        except Exception as e:
            logger.exception(f"Error in {self.name}")
            yield ModelClientStreamingChunkEvent(
                content=str(e)+"\n\n",
                source=self.name,
            )
            await model_context.add_message(
                UserMessage(
                    content=str(e),
                    source="user",
                )
            )
            yield StopMessage(
                content=str(e),
                source=agent_name,
            )

    def _to_config(self) -> SkillAgentConfig:
        """Convert the assistant agent to a declarative config."""

        return SkillAgentConfig(
            name=self.name,
            model_client=self._model_client.dump_component(),
            tools=None,  # versionchanged:: v0.5.5  Now tools are not serialized, Cause they are part of the workbench.
            workbench=self._workbench.dump_component() if self._workbench else None,
            handoffs=list(self._handoffs.values()) if self._handoffs else None,
            model_context=self._model_context.dump_component(),
            memory=[memory.dump_component() for memory in self._memory] if self._memory else None,
            description=self.description,
            system_message=self._system_messages[0].content
            if self._system_messages and isinstance(self._system_messages[0].content, str)
            else None,
            model_client_stream=self._model_client_stream,
            reflect_on_tool_use=self._reflect_on_tool_use,
            tool_call_summary_format=self._tool_call_summary_format,
            tool_call_summary_prompt=self._tool_call_summary_prompt,
            structured_message_factory=self._structured_message_factory.dump_component()
            if self._structured_message_factory
            else None,
            metadata=self._metadata,
            db_manager_config=self._db_manager.dump_component(),
            thread_id=self._thread_id,
            user_id=self._user_id,
            skills_dir=self._skills_dir,
        )
    
    @classmethod
    def _from_config(
        cls, config: SkillAgentConfig, 
        db_manager: DatabaseManager,
        memory_function: Callable = None,
        reply_function: Callable = None,
        **kwargs,
        ) -> Self:
        """Create an assistant agent from a declarative config."""
        if config.structured_message_factory:
            structured_message_factory = StructuredMessageFactory.load_component(config.structured_message_factory)
            format_string = structured_message_factory.format_string
            output_content_type = structured_message_factory.ContentModel

        else:
            format_string = None
            output_content_type = None

        return cls(
            name=config.name,
            model_client=ChatCompletionClient.load_component(config.model_client),
            workbench=Workbench.load_component(config.workbench) if config.workbench else None,
            handoffs=config.handoffs,
            model_context=ChatCompletionContext.load_component(config.model_context) if config.model_context else None,
            tools=[BaseTool.load_component(tool) for tool in config.tools] if config.tools else None,
            memory=[Memory.load_component(memory) for memory in config.memory] if config.memory else None,
            description=config.description,
            system_message=config.system_message,
            model_client_stream=config.model_client_stream,
            reflect_on_tool_use=config.reflect_on_tool_use,
            tool_call_summary_format=config.tool_call_summary_format,
            tool_call_summary_prompt=config.tool_call_summary_prompt,
            output_content_type=output_content_type,
            output_content_type_format=format_string,
            metadata=config.metadata,
            memory_function=memory_function,
            reply_function=reply_function,
            db_manager=db_manager,
            thread_id=config.thread_id,
            user_id=config.user_id,
            skills_dir=config.skills_dir,
            **kwargs,
        )



import asyncio
from typing import (
    List, 
    Optional, 
    Dict, 
    Any, 
    Union, 
    Tuple,
    Mapping,
    Sequence,
    AsyncGenerator,
    Callable,
    )

from pydantic import BaseModel, ValidationError
import uuid

from autogen_core import (
    AgentId,
    AgentRuntime,
    AgentType,
    CancellationToken,
    ComponentBase,
    SingleThreadedAgentRuntime,
    TypeSubscription,
)

from autogen_agentchat.base import (
    ChatAgent, 
    Team,
    TaskResult, 
    Response,
    TerminationCondition,
    TaskRunner,
    )
from typing import Any, Callable, List, Mapping
from loguru import logger

from autogen_core import AgentRuntime, Component, ComponentModel, CancellationToken
from pydantic import BaseModel

from autogen_agentchat.base import ChatAgent, TerminationCondition
# from autogen_agentchat.messages import BaseAgentEvent, BaseChatMessage, HandoffMessage, MessageFactory
from autogen_agentchat.state import SwarmManagerState
from autogen_agentchat.teams._group_chat._events import (
    GroupChatAgentResponse,
    GroupChatRequestPublish,
    GroupChatStart,
    GroupChatTermination,
)
from autogen_agentchat.messages import (
    BaseAgentEvent,
    BaseChatMessage,
    AgentEvent,
    ChatMessage,
    HandoffMessage,
    MemoryQueryEvent,
    ModelClientStreamingChunkEvent,
    TextMessage,
    ToolCallExecutionEvent,
    ToolCallRequestEvent,
    ToolCallSummaryMessage,
    UserInputRequestedEvent,
    ThoughtEvent,
    StructuredMessageFactory,
    MultiModalMessage,
    Image,
    # MessageFactory,
    StructuredMessage,
    StopMessage,
)
from drsai.modules.managers.messages.agent_messages import (
    AgentLongTaskMessage, 
    LongTaskQueryMessage,
    ToolLongTaskEvent,
    AgentLogEvent,
    DrSaiMessageFactory,
    )
from autogen_agentchat.teams._group_chat._events import (
    GroupChatAgentResponse,
    GroupChatError,
    GroupChatMessage,
    GroupChatPause,
    GroupChatRequestPublish,
    GroupChatReset,
    GroupChatResume,
    GroupChatStart,
    GroupChatTermination,
    SerializableException,
    )
from drsai.modules.managers.messages.groupchat_messages import (
    GroupChatAgentLongTask,
    GroupChatLazyInit,
    GroupChatClose
    )

# from autogen_agentchat.teams import BaseGroupChat
# from autogen_agentchat.teams._group_chat._base_group_chat_manager import BaseGroupChatManager
# from drsai.modules.groupchat.ag_base_group_chat import AGGroupChat, AGBaseGroupChatManager
from drsai.modules.groupchat.drsai_base_group_chat import DrSaiBaseGroupChat
from drsai.modules.groupchat.drsai_base_group_chat_manager import DrSaiBaseGroupChatManager
from drsai.modules.managers.database import DatabaseManager

class AGSwarmGroupChatManager(DrSaiBaseGroupChatManager):
    """A group chat manager that selects the next speaker based on handoff message only."""

    def __init__(
        self,
        name: str,
        group_topic_type: str,
        output_topic_type: str,
        participant_topic_types: List[str],
        participant_names: List[str],
        participant_descriptions: List[str],
        output_message_queue: asyncio.Queue[BaseAgentEvent | BaseChatMessage | GroupChatTermination],
        termination_condition: TerminationCondition | None,
        max_turns: int | None,
        message_factory: DrSaiMessageFactory,
        emit_team_events: bool,
        db_manager: DatabaseManager = None,
        **kwargs: Any
    ) -> None:
        super().__init__(
            name,
            group_topic_type,
            output_topic_type,
            participant_topic_types,
            participant_names,
            participant_descriptions,
            output_message_queue,
            termination_condition,
            max_turns,
            message_factory,
            emit_team_events,
            db_manager=db_manager,
            **kwargs
        )
        self._current_speaker = self._participant_names[0]

    async def validate_group_state(self, messages: List[BaseChatMessage] | None) -> None:
        """Validate the start messages for the group chat."""
        # Check if any of the start messages is a handoff message.
        if messages:
            for message in messages:
                if isinstance(message, HandoffMessage):
                    if message.target not in self._participant_names:
                        raise ValueError(
                            f"The target {message.target} is not one of the participants {self._participant_names}. "
                            "If you are resuming Swarm with a new HandoffMessage make sure to set the target to a valid participant as the target."
                        )
                    return

        # Check if there is a handoff message in the thread that is not targeting a valid participant.
        for existing_message in reversed(self._message_thread):
            if isinstance(existing_message, HandoffMessage):
                if existing_message.target not in self._participant_names:
                    raise ValueError(
                        f"The existing handoff target {existing_message.target} is not one of the participants {self._participant_names}. "
                        "If you are resuming Swarm with a new task make sure to include in your task "
                        "a HandoffMessage with a valid participant as the target. For example, if you are "
                        "resuming from a HandoffTermination, make sure the new task is a HandoffMessage "
                        "with a valid participant as the target."
                    )
                # The latest handoff message should always target a valid participant.
                # Do not look past the latest handoff message.
                return

    async def reset(self) -> None:
        self._current_turn = 0
        self._message_thread.clear()
        if self._termination_condition is not None:
            await self._termination_condition.reset()
        self._current_speaker = self._participant_names[0]

    async def select_speaker(self, thread: List[BaseAgentEvent | BaseChatMessage]) -> str:
        """Select a speaker from the participants based on handoff message.
        Looks for the last handoff message in the thread to determine the next speaker."""
        if len(thread) == 0:
            return self._current_speaker
        for message in reversed(thread):
            if isinstance(message, HandoffMessage):
                self._current_speaker = message.target
                # The latest handoff message should always target a valid participant.
                assert self._current_speaker in self._participant_names
                return self._current_speaker
        return self._current_speaker

    async def save_state(self) -> Mapping[str, Any]:
        state = SwarmManagerState(
            message_thread=[msg.dump() for msg in self._message_thread],
            current_turn=self._current_turn,
            current_speaker=self._current_speaker,
        )
        return state.model_dump()

    async def load_state(self, state: Mapping[str, Any]) -> None:
        swarm_state = SwarmManagerState.model_validate(state)
        self._message_thread = [self._message_factory.create(message) for message in swarm_state.message_thread]
        self._current_turn = swarm_state.current_turn
        self._current_speaker = swarm_state.current_speaker
    
    async def pause(self, cancellation_token: CancellationToken|None = None, **kwargs) -> None:
        """Pause the group chat manager."""
        logger.info(f"Pausing DrSaiSwarmGroupChatManager...")
        self._is_paused = True

    async def resume(self, cancellation_token: CancellationToken|None = None, **kwargs) -> None:
        """Resume the group chat manager."""
        self._is_paused = False

    async def close(self, cancellation_token: CancellationToken|None = None, **kwargs) -> None:
        """Close any resources."""
        self._is_paused = True
        logger.info(f"Closing DrSaiSwarmGroupChatManager...")



class AGSwarmConfig(BaseModel):
    """The declarative configuration for Swarm."""

    participants: List[ComponentModel]
    termination_condition: ComponentModel | None = None
    max_turns: int | None = None
    emit_team_events: bool = False


class AGSwarm(DrSaiBaseGroupChat, Component[AGSwarmConfig]):
    """A group chat team that selects the next speaker based on handoff message only.

    The first participant in the list of participants is the initial speaker.
    The next speaker is selected based on the :class:`~autogen_agentchat.messages.HandoffMessage` message
    sent by the current speaker. If no handoff message is sent, the current speaker
    continues to be the speaker.

    Args:
        participants (List[ChatAgent]): The agents participating in the group chat. The first agent in the list is the initial speaker.
        termination_condition (TerminationCondition, optional): The termination condition for the group chat. Defaults to None.
            Without a termination condition, the group chat will run indefinitely.
        max_turns (int, optional): The maximum number of turns in the group chat before stopping. Defaults to None, meaning no limit.
        custom_message_types (List[type[BaseAgentEvent | BaseChatMessage]], optional): A list of custom message types that will be used in the group chat.
            If you are using custom message types or your agents produces custom message types, you need to specify them here.
            Make sure your custom message types are subclasses of :class:`~autogen_agentchat.messages.BaseAgentEvent` or :class:`~autogen_agentchat.messages.BaseChatMessage`.
        emit_team_events (bool, optional): Whether to emit team events through :meth:`BaseGroupChat.run_stream`. Defaults to False.

    Basic example:

        .. code-block:: python

            import asyncio
            from autogen_ext.models.openai import OpenAIChatCompletionClient
            from autogen_agentchat.agents import AssistantAgent
            from autogen_agentchat.teams import Swarm
            from autogen_agentchat.conditions import MaxMessageTermination


            async def main() -> None:
                model_client = OpenAIChatCompletionClient(model="gpt-4o")

                agent1 = AssistantAgent(
                    "Alice",
                    model_client=model_client,
                    handoffs=["Bob"],
                    system_message="You are Alice and you only answer questions about yourself.",
                )
                agent2 = AssistantAgent(
                    "Bob", model_client=model_client, system_message="You are Bob and your birthday is on 1st January."
                )

                termination = MaxMessageTermination(3)
                team = Swarm([agent1, agent2], termination_condition=termination)

                stream = team.run_stream(task="What is bob's birthday?")
                async for message in stream:
                    print(message)


            asyncio.run(main())


    Using the :class:`~autogen_agentchat.conditions.HandoffTermination` for human-in-the-loop handoff:

        .. code-block:: python

            import asyncio
            from autogen_ext.models.openai import OpenAIChatCompletionClient
            from autogen_agentchat.agents import AssistantAgent
            from autogen_agentchat.teams import Swarm
            from autogen_agentchat.conditions import HandoffTermination, MaxMessageTermination
            from autogen_agentchat.ui import Console
            from autogen_agentchat.messages import HandoffMessage


            async def main() -> None:
                model_client = OpenAIChatCompletionClient(model="gpt-4o")

                agent = AssistantAgent(
                    "Alice",
                    model_client=model_client,
                    handoffs=["user"],
                    system_message="You are Alice and you only answer questions about yourself, ask the user for help if needed.",
                )
                termination = HandoffTermination(target="user") | MaxMessageTermination(3)
                team = Swarm([agent], termination_condition=termination)

                # Start the conversation.
                await Console(team.run_stream(task="What is bob's birthday?"))

                # Resume with user feedback.
                await Console(
                    team.run_stream(
                        task=HandoffMessage(source="user", target="Alice", content="Bob's birthday is on 1st January.")
                    )
                )


            asyncio.run(main())
    """

    component_config_schema = AGSwarmConfig
    component_provider_override = "drsai.AGSwarm"

    # TODO: Add * to the constructor to separate the positional parameters from the kwargs.
    # This may be a breaking change so let's wait until a good time to do it.
    def __init__(
        self,
        participants: List[ChatAgent],
        termination_condition: TerminationCondition | None = None,
        max_turns: int | None = None,
        runtime: AgentRuntime | None = None,
        custom_message_types: List[type[BaseAgentEvent | BaseChatMessage]] | None = None,
        emit_team_events: bool = False,
        db_manager: DatabaseManager = None,
        **kwargs: Any
    ) -> None:
        super().__init__(
            participants,
            group_chat_manager_name="AGSwarmGroupChatManager",
            group_chat_manager_class=AGSwarmGroupChatManager,
            termination_condition=termination_condition,
            max_turns=max_turns,
            runtime=runtime,
            custom_message_types=custom_message_types,
            emit_team_events=emit_team_events,
            db_manager=db_manager,
            **kwargs
        )
        # The first participant must be able to produce handoff messages.
        first_participant = self._participants[0]
        if HandoffMessage not in first_participant.produced_message_types:
            raise ValueError("The first participant must be able to produce a handoff messages.")

    def _create_group_chat_manager_factory(
        self,
        name: str,
        group_topic_type: str,
        output_topic_type: str,
        participant_topic_types: List[str],
        participant_names: List[str],
        participant_descriptions: List[str],
        output_message_queue: asyncio.Queue[BaseAgentEvent | BaseChatMessage | GroupChatTermination],
        termination_condition: TerminationCondition | None,
        max_turns: int | None,
        message_factory: DrSaiMessageFactory,
        **kwargs: Any
    ) -> Callable[[], AGSwarmGroupChatManager]:
        def _factory() -> AGSwarmGroupChatManager:
            return AGSwarmGroupChatManager(
                name,
                group_topic_type,
                output_topic_type,
                participant_topic_types,
                participant_names,
                participant_descriptions,
                output_message_queue,
                termination_condition,
                max_turns,
                message_factory,
                self._emit_team_events,
                db_manager=self._db_manager,
                **kwargs, 
            )

        return _factory
    
    async def pause(self, cancellation_token: CancellationToken|None = None, **kwargs) -> None:
        """Pause the group chat."""
        orchestrator = await self._runtime.try_get_underlying_agent_instance(
            AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
            type=AGSwarmGroupChatManager,
        )
        await orchestrator.pause()
        for agent in self._participants:
            if hasattr(agent, "pause"):
                await agent.pause()  # type: ignore

    async def resume(self, cancellation_token: CancellationToken|None = None, **kwargs) -> None:
        """Resume the group chat."""
        orchestrator = await self._runtime.try_get_underlying_agent_instance(
            AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
            type=AGSwarmGroupChatManager,
        )
        await orchestrator.resume()
        for agent in self._participants:
            if hasattr(agent, "resume"):
                await agent.resume()  # type: ignore

    async def lazy_init(self, cancellation_token: CancellationToken|None = None, **kwargs) -> None:
        """Initialize any lazy-loaded components."""
        for agent in self._participants:
            if hasattr(agent, "lazy_init"):
                await agent.lazy_init()  # type: ignore

    async def close(self, cancellation_token: CancellationToken|None = None, **kwargs) -> None:
        """Close all resources."""
        # Prepare a list of closable agents
        closable_agents: List[AGSwarmGroupChatManager | ChatAgent] = [
            agent for agent in self._participants if hasattr(agent, "close")
        ]
        # Check if we can close the orchestrator
        orchestrator = await self._runtime.try_get_underlying_agent_instance(
            AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
            type=AGSwarmGroupChatManager,
        )
        if hasattr(orchestrator, "close"):
            closable_agents.append(orchestrator)

        # Close all closable agents concurrently
        await asyncio.gather(
            *(agent.close() for agent in closable_agents), return_exceptions=True
        )

    async def _get_history_messages(
        self,
    ) -> List[BaseChatMessage]:
        
        orchestrator = await self._runtime.try_get_underlying_agent_instance(
            AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
            type=AGSwarmGroupChatManager,
        )

        return orchestrator._message_thread
        
    async def run_stream(
        self,
        *,
        task: str | BaseChatMessage | Sequence[BaseChatMessage] | None = None,
        cancellation_token: CancellationToken | None = None,
    ) -> AsyncGenerator[BaseAgentEvent | BaseChatMessage | TaskResult, None]:
        """Run the team and produces a stream of messages and the final result
        of the type :class:`~autogen_agentchat.base.TaskResult` as the last item in the stream. Once the
        team is stopped, the termination condition is reset.

        .. note::

            If an agent produces :class:`~autogen_agentchat.messages.ModelClientStreamingChunkEvent`,
            the message will be yielded in the stream but it will not be included in the
            :attr:`~autogen_agentchat.base.TaskResult.messages`.

        Args:
            task (str | BaseChatMessage | Sequence[BaseChatMessage] | None): The task to run the team with. Can be a string, a single :class:`BaseChatMessage` , or a list of :class:`BaseChatMessage`.
            cancellation_token (CancellationToken | None): The cancellation token to kill the task immediately.
                Setting the cancellation token potentially put the team in an inconsistent state,
                and it may not reset the termination condition.
                To gracefully stop the team, use :class:`~autogen_agentchat.conditions.ExternalTermination` instead.

        Returns:
            stream: an :class:`~collections.abc.AsyncGenerator` that yields :class:`~autogen_agentchat.messages.BaseAgentEvent`, :class:`~autogen_agentchat.messages.BaseChatMessage`, and the final result :class:`~autogen_agentchat.base.TaskResult` as the last item in the stream.
        """

        # Create the messages list if the task is a string or a chat message.
        messages: List[BaseChatMessage] | None = None
        if task is None:
            pass
        elif isinstance(task, str):
            messages = [TextMessage(content=task, source="user")]
        elif isinstance(task, BaseChatMessage):
            messages = [task]
        elif isinstance(task, list):
            if not task:
                raise ValueError("Task list cannot be empty.")
            messages = []
            for msg in task:
                if not isinstance(msg, BaseChatMessage):
                    raise ValueError("All messages in task list must be valid BaseChatMessage types")
                messages.append(msg)
        else:
            raise ValueError("Task must be a string, a BaseChatMessage, or a list of BaseChatMessage.")
        # Check if the messages types are registered with the message factory.
        if messages is not None:
            for msg in messages:
                if not self._message_factory.is_registered(msg.__class__):
                    raise ValueError(
                        f"Message type {msg.__class__} is not registered with the message factory. "
                        "Please register it with the message factory by adding it to the "
                        "custom_message_types list when creating the team."
                    )
        
        # Set the cancellation token.
        if cancellation_token is None:
            cancellation_token = CancellationToken()
        self._cancellation_token = cancellation_token

        if self._is_running:
            raise ValueError("The team is already running, it cannot run again until it is stopped.")
        self._is_running = True

        if self._embedded_runtime:
            # Start the embedded runtime.
            assert isinstance(self._runtime, SingleThreadedAgentRuntime)
            self._runtime.start()

        if not self._initialized:
            await self._init(self._runtime)

        shutdown_task: asyncio.Task[None] | None = None
        if self._embedded_runtime:

            async def stop_runtime() -> None:
                assert isinstance(self._runtime, SingleThreadedAgentRuntime)
                try:
                    # This will propagate any exceptions raised.
                    await self._runtime.stop_when_idle()
                    # Put a termination message in the queue to indicate that the group chat is stopped for whatever reason
                    # but not due to an exception.
                    await self._output_message_queue.put(
                        GroupChatTermination(
                            message=StopMessage(
                                content="The group chat is stopped.", source=self._group_chat_manager_name
                            )
                        )
                    )
                except Exception as e:
                    # Stop the consumption of messages and end the stream.
                    # NOTE: we also need to put a GroupChatTermination event here because when the runtime
                    # has an exception, the group chat manager may not be able to put a GroupChatTermination event in the queue.
                    # This may not be necessary if the group chat manager is able to handle the exception and put the event in the queue.
                    await self._output_message_queue.put(
                        GroupChatTermination(
                            message=StopMessage(
                                content="An exception occurred in the runtime.", source=self._group_chat_manager_name
                            ),
                            error=SerializableException.from_exception(e),
                        )
                    )

            # Create a background task to stop the runtime when the group chat
            # is stopped or has an exception.
            shutdown_task = asyncio.create_task(stop_runtime())

        try:
            history_messages = await self._get_history_messages()
            if history_messages:
                if isinstance(history_messages[-1], HandoffMessage):
                    messages[-1] = HandoffMessage(
                        source="user",
                        target=history_messages[-1].source,
                        content=messages[-1].content,
                    )
            # Run the team by sending the start message to the group chat manager.
            # The group chat manager will start the group chat by relaying the message to the participants
            # and the group chat manager.
            await self._runtime.send_message(
                GroupChatStart(messages=messages),
                recipient=AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
                cancellation_token=cancellation_token,
            )
            # Collect the output messages in order.
            output_messages: List[BaseAgentEvent | BaseChatMessage] = []
            stop_reason: str | None = None
            # Yield the messsages until the queue is empty.
            while True:
                message_future = asyncio.ensure_future(self._output_message_queue.get())
                if cancellation_token is not None:
                    cancellation_token.link_future(message_future)
                # Wait for the next message, this will raise an exception if the task is cancelled.
                message = await message_future
                if isinstance(message, GroupChatTermination):
                    # If the message contains an error, we need to raise it here.
                    # This will stop the team and propagate the error.
                    if message.error is not None:
                        raise RuntimeError(str(message.error))
                    stop_reason = message.message.content
                    break
                yield message
                if isinstance(message, HandoffMessage):
                    # Skip the model client streaming start events.
                    yield ModelClientStreamingChunkEvent(source=message.source, content=message.content)
                    if message.target == "user":
                        break
                if isinstance(message, ModelClientStreamingChunkEvent):
                    # Skip the model client streaming chunk events.
                    continue
                output_messages.append(message)

            # Yield the final result.
            yield TaskResult(messages=output_messages, stop_reason=stop_reason)

        finally:
            try:
                if shutdown_task is not None:
                    # Wait for the shutdown task to finish.
                    # This will propagate any exceptions raised.
                    await shutdown_task
            finally:
                # Clear the output message queue.
                while not self._output_message_queue.empty():
                    self._output_message_queue.get_nowait()

                # Indicate that the team is no longer running.
                self._is_running = False

    def _to_config(self) -> AGSwarmConfig:
        participants = [participant.dump_component() for participant in self._participants]
        termination_condition = self._termination_condition.dump_component() if self._termination_condition else None
        return AGSwarmConfig(
            participants=participants,
            termination_condition=termination_condition,
            max_turns=self._max_turns,
            emit_team_events=self._emit_team_events,
        )

    @classmethod
    def _from_config(cls, config: AGSwarmConfig) -> "AGSwarm":
        participants = [ChatAgent.load_component(participant) for participant in config.participants]
        termination_condition = (
            TerminationCondition.load_component(config.termination_condition) if config.termination_condition else None
        )
        return cls(
            participants,
            termination_condition=termination_condition,
            max_turns=config.max_turns,
            emit_team_events=config.emit_team_events,
        )



# from dataclasses import Field
from datetime import datetime
from typing import Any, Dict, List, Literal, Optional, Sequence

from autogen_agentchat.base import TaskResult
from autogen_agentchat.messages import BaseChatMessage, BaseTextChatMessage
from autogen_core import ComponentModel
from pydantic import BaseModel, field_serializer


class MessageConfig(BaseModel):
    source: str
    content: str | BaseChatMessage | Sequence[BaseChatMessage] | None
    message_type: Optional[str] = "text"


class TeamResult(BaseModel):
    task_result: TaskResult
    usage: str
    duration: float
    files: Optional[List[dict[str, Any]]] = None


class LLMCallEventMessage(BaseTextChatMessage):
    source: str = "llm_call_event"
    content: str


class MessageMeta(BaseModel):
    task: Optional[str] = None
    task_result: Optional[TaskResult] = None
    summary_method: Optional[str] = "last"
    files: Optional[List[dict[str, Any]]] = None
    time: Optional[datetime] = None
    log: Optional[List[dict[str, Any]]] = None
    usage: Optional[List[dict[str, Any]]] = None


class GalleryMetadata(BaseModel):
    author: str
    # created_at: datetime = Field(default_factory=datetime.now)
    # updated_at: datetime = Field(default_factory=datetime.now)
    version: str
    description: Optional[str] = None
    tags: Optional[List[str]] = None
    license: Optional[str] = None
    homepage: Optional[str] = None
    category: Optional[str] = None
    last_synced: Optional[datetime] = None

    @field_serializer("last_synced")
    def serialize_last_synced(cls, value: Optional[datetime]) -> Optional[str]:
        if isinstance(value, datetime):
            return value.isoformat()
        else:
            return None


class GalleryComponents(BaseModel):
    agents: List[ComponentModel]
    models: List[ComponentModel]
    tools: List[ComponentModel]
    terminations: List[ComponentModel]
    teams: List[ComponentModel]


class GalleryConfig(BaseModel):
    id: str
    name: str
    url: Optional[str] = None
    metadata: GalleryMetadata
    components: GalleryComponents


class EnvironmentVariable(BaseModel):
    name: str
    value: str
    type: Literal["string", "number", "boolean", "secret"] = "string"
    description: Optional[str] = None
    required: bool = False


class UISettings(BaseModel):
    show_llm_call_events: bool = False
    expanded_messages_by_default: bool = True
    show_agent_flow_by_default: bool = True


class SettingsConfig(BaseModel):
    environment: List[EnvironmentVariable] = []
    ui: UISettings = UISettings()


# web request/response data models


class Response(BaseModel):
    message: str
    status: bool
    data: Optional[Any] = None


class SocketMessage(BaseModel):
    connection_id: str
    data: Dict[str, Any]
    type: str





import asyncio

from typing import (
    Any, 
    List, 
    Callable, 
    Mapping,
    Dict,
    AsyncGenerator,
    Sequence,
    cast,
    )

from autogen_core import (
    AgentId,
    CancellationToken,
    SingleThreadedAgentRuntime,
    AgentRuntime
)

from autogen_agentchat.base import ChatAgent, TaskResult, TerminationCondition
from autogen_agentchat.messages import (
    BaseAgentEvent, 
    AgentEvent, 
    BaseChatMessage, 
    ChatMessage, 
    MessageFactory,
    ModelClientStreamingChunkEvent, 
    StopMessage,
    TextMessage)
from autogen_agentchat.teams._group_chat._events import (
    GroupChatStart, 
    GroupChatTermination,
    SerializableException,
    )
from autogen_agentchat.teams._group_chat._sequential_routed_agent import SequentialRoutedAgent
from autogen_agentchat.teams._group_chat._base_group_chat_manager import BaseGroupChatManager
from autogen_agentchat.teams import BaseGroupChat
from autogen_agentchat.state import BaseState, TeamState

from drsai.modules.managers.database import DatabaseManager



class BaseManagerState(BaseState):
    """The state of the RoundRobinGroupChatManager."""

    message_thread: List[Dict[str, Any]] = []
    current_turn: int = 0
    is_paused: bool = False

class AGBaseGroupChatManager(BaseGroupChatManager):

    def __init__(
        self,
        name: str,
        group_topic_type: str,
        output_topic_type: str,
        participant_topic_types: List[str],
        participant_names: List[str],
        participant_descriptions: List[str],
        output_message_queue: asyncio.Queue[BaseAgentEvent | BaseChatMessage | GroupChatTermination],
        termination_condition: TerminationCondition | None,
        max_turns: int | None,
        message_factory: MessageFactory,
        emit_team_events: bool = False,
        db_manager: DatabaseManager = None,
        **kwargs: Any
    ):
        
        super().__init__(
            name=name,
            group_topic_type=group_topic_type,
            output_topic_type=output_topic_type,
            participant_topic_types=participant_topic_types,
            participant_names=participant_names,
            participant_descriptions=participant_descriptions,
            output_message_queue=output_message_queue,
            termination_condition=termination_condition,
            max_turns=max_turns,
            message_factory=message_factory,
            emit_team_events=emit_team_events,
        )
        self._db_manager: DatabaseManager = db_manager
        
        self._is_paused = False

    async def pause(self) -> None:
        """Pause the group chat manager."""
        self._is_paused = True

    async def resume(self) -> None:
        """Resume the group chat manager."""
        self._is_paused = False

    async def close(self) -> None:
        """Close any resources."""
        pass

    async def reset(self) -> None:
        self._current_turn = 0
        self._message_thread.clear()
        if self._termination_condition is not None:
            await self._termination_condition.reset()
        self._is_paused = False

    async def save_state(self) -> Mapping[str, Any]:
        state = BaseManagerState(
            message_thread=[
                cast(Dict[str, Any], message.dump()) for message in self._message_thread
            ],
            current_turn=0,
            is_paused=False,
        )
        return state.model_dump()
    
    async def load_state(self, state: Mapping[str, Any]) -> None:
        base_state = BaseManagerState.model_validate(state)
        self._message_thread = [
            self._message_factory.create(message)
            for message in base_state.message_thread
        ]
        self._current_turn = base_state.current_turn
        self._is_paused = base_state.is_paused
    
    async def validate_group_state(
        self, messages: List[BaseChatMessage] | None
    ) -> None:
        pass

class AGGroupChat(BaseGroupChat):

    component_type = "team"

    def __init__(
        self,
        participants: List[ChatAgent],
        group_chat_manager_name: str,
        group_chat_manager_class: type[SequentialRoutedAgent],
        termination_condition: TerminationCondition | None = None,
        max_turns: int | None = None,
        runtime: AgentRuntime | None = None,
        custom_message_types: List[type[BaseAgentEvent | BaseChatMessage]] | None = None,
        emit_team_events: bool = False,
        db_manager: DatabaseManager = None,
        **kwargs: Any
    ):
        super().__init__(
            participants=participants,
            group_chat_manager_name=group_chat_manager_name,
            group_chat_manager_class=group_chat_manager_class,
            termination_condition=termination_condition,
            max_turns=max_turns,
            runtime=runtime,
            custom_message_types=custom_message_types,
            emit_team_events=emit_team_events,
            )
        self._db_manager: DatabaseManager = db_manager
        self._is_paused = False

    @property
    def participants(self) -> List[ChatAgent]:
        """
        Get the list of participants in the group chat.

        Returns:
            List[ChatAgent]: The list of participants.
        """
        return self._participants
    
    def _create_group_chat_manager_factory(
        self,
        name: str,
        group_topic_type: str,
        output_topic_type: str,
        participant_topic_types: List[str],
        participant_names: List[str],
        participant_descriptions: List[str],
        output_message_queue: asyncio.Queue[BaseAgentEvent | BaseChatMessage | GroupChatTermination],
        termination_condition: TerminationCondition | None,
        max_turns: int | None,
        message_factory: MessageFactory,
        **kwargs: Any
    ) -> Callable[[], AGBaseGroupChatManager]:
        def _factory() -> AGBaseGroupChatManager:
            return AGBaseGroupChatManager(
                name = name,
                group_topic_type = group_topic_type,
                output_topic_type = output_topic_type,
                participant_topic_types = participant_topic_types,
                participant_names = participant_names,
                participant_descriptions = participant_descriptions,
                output_message_queue = output_message_queue,
                termination_condition = termination_condition,
                max_turns = max_turns,
                message_factory = message_factory,
                db_manager = self._db_manager,
                **kwargs, 
            )

        return _factory
    
    async def run_stream(
        self,
        *,
        task: str | BaseChatMessage | Sequence[BaseChatMessage] | None = None,
        cancellation_token: CancellationToken | None = None,
    ) -> AsyncGenerator[BaseAgentEvent | BaseChatMessage | TaskResult, None]:
        """Run the team and produces a stream of messages and the final result
        of the type :class:`~autogen_agentchat.base.TaskResult` as the last item in the stream. Once the
        team is stopped, the termination condition is reset.

        .. note::

            If an agent produces :class:`~autogen_agentchat.messages.ModelClientStreamingChunkEvent`,
            the message will be yielded in the stream but it will not be included in the
            :attr:`~autogen_agentchat.base.TaskResult.messages`.

        Args:
            task (str | BaseChatMessage | Sequence[BaseChatMessage] | None): The task to run the team with. Can be a string, a single :class:`BaseChatMessage` , or a list of :class:`BaseChatMessage`.
            cancellation_token (CancellationToken | None): The cancellation token to kill the task immediately.
                Setting the cancellation token potentially put the team in an inconsistent state,
                and it may not reset the termination condition.
                To gracefully stop the team, use :class:`~autogen_agentchat.conditions.ExternalTermination` instead.

        Returns:
            stream: an :class:`~collections.abc.AsyncGenerator` that yields :class:`~autogen_agentchat.messages.BaseAgentEvent`, :class:`~autogen_agentchat.messages.BaseChatMessage`, and the final result :class:`~autogen_agentchat.base.TaskResult` as the last item in the stream.

        Example using the :class:`~autogen_agentchat.teams.RoundRobinGroupChat` team:

        .. code-block:: python

            import asyncio
            from autogen_agentchat.agents import AssistantAgent
            from autogen_agentchat.conditions import MaxMessageTermination
            from autogen_agentchat.teams import RoundRobinGroupChat
            from autogen_ext.models.openai import OpenAIChatCompletionClient


            async def main() -> None:
                model_client = OpenAIChatCompletionClient(model="gpt-4o")

                agent1 = AssistantAgent("Assistant1", model_client=model_client)
                agent2 = AssistantAgent("Assistant2", model_client=model_client)
                termination = MaxMessageTermination(3)
                team = RoundRobinGroupChat([agent1, agent2], termination_condition=termination)

                stream = team.run_stream(task="Count from 1 to 10, respond one at a time.")
                async for message in stream:
                    print(message)

                # Run the team again without a task to continue the previous task.
                stream = team.run_stream()
                async for message in stream:
                    print(message)


            asyncio.run(main())


        Example using the :class:`~autogen_core.CancellationToken` to cancel the task:

        .. code-block:: python

            import asyncio
            from autogen_agentchat.agents import AssistantAgent
            from autogen_agentchat.conditions import MaxMessageTermination
            from autogen_agentchat.ui import Console
            from autogen_agentchat.teams import RoundRobinGroupChat
            from autogen_core import CancellationToken
            from autogen_ext.models.openai import OpenAIChatCompletionClient


            async def main() -> None:
                model_client = OpenAIChatCompletionClient(model="gpt-4o")

                agent1 = AssistantAgent("Assistant1", model_client=model_client)
                agent2 = AssistantAgent("Assistant2", model_client=model_client)
                termination = MaxMessageTermination(3)
                team = RoundRobinGroupChat([agent1, agent2], termination_condition=termination)

                cancellation_token = CancellationToken()

                # Create a task to run the team in the background.
                run_task = asyncio.create_task(
                    Console(
                        team.run_stream(
                            task="Count from 1 to 10, respond one at a time.",
                            cancellation_token=cancellation_token,
                        )
                    )
                )

                # Wait for 1 second and then cancel the task.
                await asyncio.sleep(1)
                cancellation_token.cancel()

                # This will raise a cancellation error.
                await run_task


            asyncio.run(main())

        """
        
        if self._is_running:
            yield ModelClientStreamingChunkEvent(content="The team is already running, it cannot run again until it is stopped.", source="Manager")
            textmessage=TextMessage(content="The team is already running, it cannot run again until it is stopped.", source="Manager")
            yield textmessage
            yield TaskResult(messages = [textmessage], stop_reason="stop")
            return
        
        if self._is_paused:
            yield ModelClientStreamingChunkEvent(content="The team is paused, please resume it before running again.", source="Manager")
            textmessage=TextMessage(content="The team is paused, please resume it before running again.", source="Manager")
            yield textmessage
            yield TaskResult(messages = [textmessage], stop_reason="stop")
            return

        # Create the messages list if the task is a string or a chat message.
        messages: List[BaseChatMessage] | None = None
        if task is None:
            pass
        elif isinstance(task, str):
            messages = [TextMessage(content=task, source="user", metadata={"internal": "yes"})]
        elif isinstance(task, BaseChatMessage):
            task.metadata["internal"] = "yes"
            messages = [task]
        elif isinstance(task, list):
            if not task:
                raise ValueError("Task list cannot be empty.")
            messages = []
            for msg in task:
                msg.metadata["internal"] = "yes"
                if not isinstance(msg, BaseChatMessage):
                    raise ValueError("All messages in task list must be valid BaseChatMessage types")
                messages.append(msg)
        else:
            raise ValueError("Task must be a string, a BaseChatMessage, or a list of BaseChatMessage.")
        # Check if the messages types are registered with the message factory.
        if messages is not None:
            for msg in messages:
                if not self._message_factory.is_registered(msg.__class__):
                    raise ValueError(
                        f"Message type {msg.__class__} is not registered with the message factory. "
                        "Please register it with the message factory by adding it to the "
                        "custom_message_types list when creating the team."
                    )

        if self._is_running:
            raise ValueError("The team is already running, it cannot run again until it is stopped.")
        self._is_running = True

        if self._embedded_runtime:
            # Start the embedded runtime.
            assert isinstance(self._runtime, SingleThreadedAgentRuntime)
            self._runtime.start()

        if not self._initialized:
            await self._init(self._runtime)

        shutdown_task: asyncio.Task[None] | None = None
        if self._embedded_runtime:

            async def stop_runtime() -> None:
                assert isinstance(self._runtime, SingleThreadedAgentRuntime)
                try:
                    # This will propagate any exceptions raised.
                    await self._runtime.stop_when_idle()
                    # Put a termination message in the queue to indicate that the group chat is stopped for whatever reason
                    # but not due to an exception.
                    await self._output_message_queue.put(
                        GroupChatTermination(
                            message=StopMessage(
                                content="The group chat is stopped.", source=self._group_chat_manager_name
                            )
                        )
                    )
                except Exception as e:
                    # Stop the consumption of messages and end the stream.
                    # NOTE: we also need to put a GroupChatTermination event here because when the runtime
                    # has an exception, the group chat manager may not be able to put a GroupChatTermination event in the queue.
                    # This may not be necessary if the group chat manager is able to handle the exception and put the event in the queue.
                    await self._output_message_queue.put(
                        GroupChatTermination(
                            message=StopMessage(
                                content="An exception occurred in the runtime.", source=self._group_chat_manager_name
                            ),
                            error=SerializableException.from_exception(e),
                        )
                    )

            # Create a background task to stop the runtime when the group chat
            # is stopped or has an exception.
            shutdown_task = asyncio.create_task(stop_runtime())

        try:
            # Run the team by sending the start message to the group chat manager.
            # The group chat manager will start the group chat by relaying the message to the participants
            # and the group chat manager.
            await self._runtime.send_message(
                GroupChatStart(messages=messages),
                recipient=AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
                cancellation_token=cancellation_token,
            )
            # Collect the output messages in order.
            output_messages: List[BaseAgentEvent | BaseChatMessage] = []
            stop_reason: str | None = None
            # Yield the messsages until the queue is empty.
            while True:
                message_future = asyncio.ensure_future(self._output_message_queue.get())
                if cancellation_token is not None:
                    cancellation_token.link_future(message_future)
                # Wait for the next message, this will raise an exception if the task is cancelled.
                message = await message_future
                if isinstance(message, GroupChatTermination):
                    # If the message contains an error, we need to raise it here.
                    # This will stop the team and propagate the error.
                    if message.error is not None:
                        raise RuntimeError(str(message.error))
                    stop_reason = message.message.content
                    break
                yield message
                if isinstance(message, ModelClientStreamingChunkEvent):
                    # Skip the model client streaming chunk events.
                    continue
                output_messages.append(message)

            # Yield the final result.
            yield TaskResult(messages=output_messages, stop_reason=stop_reason)

        finally:
            try:
                if shutdown_task is not None:
                    # Wait for the shutdown task to finish.
                    # This will propagate any exceptions raised.
                    await shutdown_task
            finally:
                # Clear the output message queue.
                while not self._output_message_queue.empty():
                    self._output_message_queue.get_nowait()

                # Indicate that the team is no longer running.
                self._is_running = False
    
    async def pause(self) -> None:
        """Pause the group chat."""
        orchestrator = await self._runtime.try_get_underlying_agent_instance(
            AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
            type=AGBaseGroupChatManager,
        )
        await orchestrator.pause()
        for agent in self._participants:
            if hasattr(agent, "pause"):
                await agent.pause()  # type: ignore
        
        self._is_running = False
        self._is_paused = True

    async def resume(self) -> None:
        """Resume the group chat."""
        orchestrator = await self._runtime.try_get_underlying_agent_instance(
            AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
            type=AGBaseGroupChatManager,
        )
        await orchestrator.resume()
        for agent in self._participants:
            if hasattr(agent, "resume"):
                await agent.resume()  # type: ignore
        
        self._is_paused = False

    async def lazy_init(self) -> None:
        """Initialize any lazy-loaded components."""
        for agent in self._participants:
            if hasattr(agent, "lazy_init"):
                await agent.lazy_init()  # type: ignore

    async def close(self) -> None:
        """Close all resources."""
        # Prepare a list of closable agents
        closable_agents: List[AGBaseGroupChatManager | ChatAgent] = [
            agent for agent in self._participants if hasattr(agent, "close")
        ]
        # Check if we can close the orchestrator
        orchestrator = await self._runtime.try_get_underlying_agent_instance(
            AgentId(type=self._group_chat_manager_topic_type, key=self._team_id),
            type=AGBaseGroupChatManager,
        )
        if hasattr(orchestrator, "close"):
            closable_agents.append(orchestrator)

        # Close all closable agents concurrently
        await asyncio.gather(
            *(agent.close() for agent in closable_agents), return_exceptions=True
        )

    async def _get_partial_state(self) -> Mapping[str, Any]:
        """Save the state of the group chat team."""
        try:
            # Save the state of the runtime. This will save the state of the participants and the group chat manager.
            agent_states: Dict[str, Mapping[str, Any]] = {}
            # Save the state of all participants.
            for name, agent_type in zip(
                self._participant_names, self._participant_topic_types, strict=True
            ):
                agent_id = AgentId(type=agent_type, key=self._team_id)
                # NOTE: We are using the runtime's save state method rather than the agent instance's
                # save_state method because we want to support saving state of remote agents.
                agent_states[name] = await self._runtime.agent_save_state(agent_id)
            # Save the state of the group chat manager.
            agent_id = AgentId(
                type=self._group_chat_manager_topic_type, key=self._team_id
            )
            agent_states[
                self._group_chat_manager_name
            ] = await self._runtime.agent_save_state(agent_id)
            return TeamState(agent_states=agent_states).model_dump()
        finally:
            # Indicate that the team is no longer running.
            self._is_running = False


"""
wechat_bot.py — 微信 ilink Bot 主循环
======================================
负责：长轮询接收消息 → 命令路由 → 调用 DrSai agent → 分段回复微信。

命令列表：
  /help            —— 显示帮助
  /newsession      —— 新建会话，开始与 agent 的全新对话
  /session         —— 列出该用户所有历史会话
  /session <id>    —— 切换到指定会话

其他文字消息 → 转发给当前会话的 DrSai agent，逐条实时发送 TextMessage 回复。
"""

import asyncio
import json
import logging
from typing import TYPE_CHECKING

from .wechat_client import AsyncWeChatAPI, MessageType, split_text
from .session_manager import SessionManager

if TYPE_CHECKING:
    from drsai.backend.run import DrSaiWorkerModel

logger = logging.getLogger(__name__)

# 等待提示：当 agent 处理时间可能较长时先发一条占位消息
THINKING_HINT = "⏳ 正在处理，请稍候..."

HELP_TEXT = """DrSai Bot 命令列表：
/help            —— 显示此帮助
/newsession      —— 新建对话（开始全新 session）
/session         —— 查看所有历史 session
/session <id>    —— 切换到指定 session（如 /session session_1）
/models          —— 列出所有可用模型
/model <模型名>   —— 使用指定模型
其他文字         —— 与当前 session 的 AI 助手对话""".strip()

# 连续失败退避阈值
BACKOFF_THRESHOLD = 3
BACKOFF_SHORT = 3
BACKOFF_LONG = 30

# 最多缓存最近 N 条消息 ID 用于去重
MAX_DEDUP_SIZE = 1000


class WeChatBot:
    """
    微信 Bot 主控类。

    - 持续长轮询拉取新消息
    - 按命令路由或转发 agent
    - 每个用户持有一把 asyncio.Lock，防止并发消息互相干扰
    """

    def __init__(
        self,
        model: "DrSaiWorkerModel",
        creds: dict,
        api_key: str,
        session_manager: SessionManager,
    ):
        self.model = model
        self.creds = creds
        self.api_key = api_key
        self.session_manager = session_manager
        self.api = AsyncWeChatAPI(
            bot_token=creds["bot_token"],
            base_url=creds.get("base_url", "https://ilinkai.weixin.qq.com"),
        )
        self._user_locks: dict[str, asyncio.Lock] = {}
        self._seen_ids: set = set()

        self._msg_metadata = {}

    # ── 主循环 ────────────────────────────────────────────────────────────────

    async def run(self) -> None:
        """长轮询主循环，持续拉取并处理新消息。"""
        from .wechat_client import load_sync_buf, save_sync_buf

        sync_buf = load_sync_buf()
        consecutive_failures = 0

        logger.info("WeChatBot 已启动，账号: %s", self.creds.get("account_id"))

        while True:
            try:
                resp = await self.api.get_updates(sync_buf or None)
            except asyncio.CancelledError:
                logger.info("WeChatBot 收到取消信号，退出。")
                break
            except Exception as e:
                consecutive_failures += 1
                wait = BACKOFF_LONG if consecutive_failures >= BACKOFF_THRESHOLD else BACKOFF_SHORT
                logger.warning("getupdates 出错（第%d次）: %s，%ds 后重试", consecutive_failures, e, wait)
                await asyncio.sleep(wait)
                continue

            consecutive_failures = 0

            # session 过期
            if resp.get("ret") == -14:
                logger.error("微信 session 已过期，Bot 暂停。请重新启动后端并扫码登录。")
                await asyncio.sleep(3600)
                continue

            # 更新游标
            new_buf = resp.get("get_updates_buf")
            if new_buf:
                sync_buf = new_buf
                save_sync_buf(sync_buf)

            # 分发消息（各消息并发处理，同一用户串行）
            tasks = []
            for msg in resp.get("msgs") or []:
                msg_id = msg.get("message_id")
                if msg_id and msg_id in self._seen_ids:
                    continue
                if msg_id:
                    self._seen_ids.add(msg_id)
                    if len(self._seen_ids) > MAX_DEDUP_SIZE:
                        to_rm = list(self._seen_ids)[: MAX_DEDUP_SIZE // 2]
                        self._seen_ids.difference_update(to_rm)
                tasks.append(asyncio.create_task(self._dispatch(msg)))

            if tasks:
                await asyncio.gather(*tasks, return_exceptions=True)

    # ── 消息分发（带用户级锁） ────────────────────────────────────────────────

    async def _dispatch(self, msg: dict) -> None:
        """确保同一用户的消息串行处理，避免乱序。"""
        if msg.get("message_type") != MessageType.USER:
            return

        user_id = msg.get("from_user_id", "")
        if not user_id:
            return

        lock = self._user_locks.setdefault(user_id, asyncio.Lock())
        async with lock:
            try:
                await self.handle_message(msg)
            except Exception as e:
                logger.exception("处理消息 %s 时出错: %s", msg.get("message_id"), e)

    # ── 消息处理 ─────────────────────────────────────────────────────────────

    async def handle_message(self, msg: dict) -> None:
        user_id = msg.get("from_user_id", "")
        context_token = msg.get("context_token", "")
        text = AsyncWeChatAPI.extract_text(msg).strip()

        logger.info("[msg#%s] from=%s text=%r", msg.get("message_id"), user_id, text[:80])

        if not text:
            await self._reply(user_id, context_token, "暂不支持此类消息，请发送文字。")
            return

        # ── 命令路由 ──────────────────────────────────────────────────────────
        if text == "/help":
            await self._reply(user_id, context_token, HELP_TEXT)
            return

        if text == "/newsession":
            chat_id = self.session_manager.new_session(user_id)
            await self._reply(user_id, context_token,
                              f"✅ 已创建新会话 {chat_id}，后续对话将在此会话中进行。")
            return

        if text == "/session":
            sessions = self.session_manager.list_sessions(user_id)
            current = self.session_manager.get_current(user_id)
            if not sessions:
                await self._reply(user_id, context_token, "暂无历史会话，发送任意消息即可自动创建。")
            else:
                lines = [f"历史会话列表（当前: {current}）："]
                for sid in sessions:
                    mark = " ←当前" if sid == current else ""
                    lines.append(f"  • {sid}{mark}")
                lines.append("\n发送 /session <id> 切换会话。")
                await self._reply(user_id, context_token, "\n".join(lines))
            return

        if text.startswith("/session "):
            target = text[9:].strip()
            if self.session_manager.switch_session(user_id, target):
                await self._reply(user_id, context_token, f"✅ 已切换到会话 {target}。")
            else:
                await self._reply(user_id, context_token,
                                  f"❌ 会话 {target!r} 不存在或不属于你，请用 /session 查看列表。")
            return

        # ── 转发给 agent ──────────────────────────────────────────────────────
        chat_id = self.session_manager.get_or_create_session(user_id)

        # 先发"处理中"占位消息
        await self._reply(user_id, context_token, THINKING_HINT)

        # 确保 agent 已初始化
        await self._ensure_agent(chat_id, user_id)

        if text.startswith("/models"):
            llm_mode_config: dict = self.model.drsai.agent_instance[chat_id]._llm_mode_config
            llm_mode_config_format = "可用的模型配置：\n"
            for k, _ in llm_mode_config.items():
                llm_mode_config_format += f"{k}\n\n"
            await self._reply(user_id, context_token, llm_mode_config_format)
            return
        
        
        if text.startswith("/model "):
            model_name = text[7:].strip()
            # self.model.drsai.agent_instance[chat_id].set_llm_mode(model_name)
            self._msg_metadata.update({"settings_config": json.dumps({"defult_config_name": model_name})})
            await self._reply(user_id, context_token, f"已切换模型为 {model_name}")
            return
        # 以 a_drsai_ui_completions 为核心，收到每条 TextMessage 立即发送
        messages = [
            {
                "type": "TextMessage",
                "source": "user",
                "content": text,
                "metadata": self._msg_metadata,
            }
        ]
        kwargs = dict(
            chat_id=chat_id,
            api_key=self.api_key,
            messages=messages,
            stream=True,
            user={"email": user_id},
        )

        replied = False
        try:
            async for line in self.model.drsai.a_drsai_ui_completions(**kwargs):
                if not isinstance(line, str):
                    continue
                line = line.strip()
                if not line.startswith("data: "):
                    continue
                try:
                    event = json.loads(line[6:])
                except json.JSONDecodeError:
                    continue

                event_type = event.get("type", "")

                if event_type == "TextMessage":
                    source = event.get("source", "")
                    content = event.get("content", "")
                    if source != "user" and content:
                        await self._reply(user_id, context_token, content)
                        replied = True

                elif event_type == "TaskResult":
                    break

        except Exception as e:
            logger.exception("agent 调用出错 (chat_id=%s): %s", chat_id, e)

        if not replied:
            await self._reply(user_id, context_token, "（Agent 未返回内容，请重试）")

        # 更新活跃时间
        self.session_manager.touch(chat_id)

    # ── 发送消息（自动分段） ──────────────────────────────────────────────────

    async def _reply(self, to_user_id: str, context_token: str, text: str) -> None:
        """将文本分段后逐条发送给用户。"""
        account_id = self.creds["account_id"]
        for chunk in split_text(text):
            try:
                await self.api.send_text(account_id, to_user_id, context_token, chunk)
            except Exception as e:
                logger.error("发送消息失败 to=%s: %s", to_user_id, e)

    # ── 确保 agent 已初始化 ───────────────────────────────────────────────────

    async def _ensure_agent(self, chat_id: str, user_id: str) -> None:
        """调用 lazy_init 确保对应 chat_id 的 agent 实例已创建。"""
        run_info = {"email": user_id}
        result = await self.model.lazy_init(
            chat_id=chat_id,
            api_key=self.api_key,
            run_info=run_info,
            stream=True,
        )
        if not result.get("status"):
            raise RuntimeError(f"lazy_init 失败: {result.get('message')}")

